@@ -26,6 +26,11 @@
 #include <gtk/gtk.h>
 #include <glib/gprintf.h>
 #include <glib/gi18n.h>
+#include <libxml/uri.h>
+
+#ifdef HAVE_OVIRT
+#include <govirt/govirt.h>
+#endif
 
 #ifdef HAVE_SPICE_GTK
 #include <spice-controller.h>
@@ -35,6 +40,9 @@
 #include "virt-viewer-session-spice.h"
 #endif
 #include "virt-viewer-app.h"
+#include "virt-viewer-auth.h"
+#include "virt-viewer-file.h"
+#include "virt-viewer-session.h"
 #include "remote-viewer.h"
 
 #ifndef G_VALUE_INIT /* see bug https://bugzilla.gnome.org/show_bug.cgi?id=654793 */
@@ -54,7 +62,7 @@ G_DEFINE_TYPE (RemoteViewer, remote_viewer, VIRT_VIEWER_TYPE_APP)
 #define GET_PRIVATE(o)                                                        \
     (G_TYPE_INSTANCE_GET_PRIVATE ((o), REMOTE_VIEWER_TYPE, RemoteViewerPrivate))
 
-#if HAVE_SPICE_GTK
+#ifdef HAVE_SPICE_GTK
 enum {
     PROP_0,
     PROP_CONTROLLER,
@@ -63,13 +71,11 @@ enum {
 #endif
 
 static gboolean remote_viewer_start(VirtViewerApp *self);
-#if HAVE_SPICE_GTK
-static int remote_viewer_activate(VirtViewerApp *self);
+#ifdef HAVE_SPICE_GTK
+static gboolean remote_viewer_activate(VirtViewerApp *self, GError **error);
 static void remote_viewer_window_added(VirtViewerApp *self, VirtViewerWindow *win);
 static void spice_foreign_menu_updated(RemoteViewer *self);
-#endif
 
-#if HAVE_SPICE_GTK
 static void
 remote_viewer_get_property (GObject *object, guint property_id,
                             GValue *value, GParamSpec *pspec)
@@ -133,26 +139,26 @@ remote_viewer_dispose (GObject *object)
 static void
 remote_viewer_class_init (RemoteViewerClass *klass)
 {
-#if HAVE_SPICE_GTK
+#ifdef HAVE_SPICE_GTK
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
 #endif
     VirtViewerAppClass *app_class = VIRT_VIEWER_APP_CLASS (klass);
 
     g_type_class_add_private (klass, sizeof (RemoteViewerPrivate));
 
-#if HAVE_SPICE_GTK
+#ifdef HAVE_SPICE_GTK
     object_class->get_property = remote_viewer_get_property;
     object_class->set_property = remote_viewer_set_property;
     object_class->dispose = remote_viewer_dispose;
 #endif
 
     app_class->start = remote_viewer_start;
-#if HAVE_SPICE_GTK
+#ifdef HAVE_SPICE_GTK
     app_class->activate = remote_viewer_activate;
     app_class->window_added = remote_viewer_window_added;
 #endif
 
-#if HAVE_SPICE_GTK
+#ifdef HAVE_SPICE_GTK
     g_object_class_install_property(object_class,
                                     PROP_CONTROLLER,
                                     g_param_spec_object("controller",
@@ -181,15 +187,16 @@ remote_viewer_init(RemoteViewer *self)
 }
 
 RemoteViewer *
-remote_viewer_new(const gchar *uri, gboolean verbose)
+remote_viewer_new(const gchar *uri, const gchar *title, gboolean verbose)
 {
     return g_object_new(REMOTE_VIEWER_TYPE,
                         "guri", uri,
                         "verbose", verbose,
+                        "title", title,
                         NULL);
 }
 
-#if HAVE_SPICE_GTK
+#ifdef HAVE_SPICE_GTK
 static void
 foreign_menu_title_changed(SpiceCtrlForeignMenu *menu G_GNUC_UNUSED,
                            GParamSpec *pspec G_GNUC_UNUSED,
@@ -233,8 +240,13 @@ static void
 spice_ctrl_do_connect(SpiceCtrlController *ctrl G_GNUC_UNUSED,
                       VirtViewerApp *self)
 {
-    if (virt_viewer_app_initial_connect(self) < 0) {
-        virt_viewer_app_simple_message_dialog(self, _("Failed to initiate connection"));
+    GError *error = NULL;
+
+    if (!virt_viewer_app_initial_connect(self, &error)) {
+        const gchar *msg = error ? error->message :
+            _("Failed to initiate connection");
+        virt_viewer_app_simple_message_dialog(self, msg);
+        g_clear_error(&error);
     }
 }
 
@@ -434,119 +446,6 @@ remote_viewer_get_spice_session(RemoteViewer *self)
     return session;
 }
 
-static gchar *
-ctrl_key_to_gtk_key(const gchar *key)
-{
-    int i;
-
-    static const struct {
-        const char *ctrl;
-        const char *gtk;
-    } keys[] = {
-        /* FIXME: right alt, right ctrl, right shift, cmds */
-        { "alt", "<Alt>" },
-        { "ralt", "<Alt>" },
-        { "rightalt", "<Alt>" },
-        { "right-alt", "<Alt>" },
-        { "lalt", "<Alt>" },
-        { "leftalt", "<Alt>" },
-        { "left-alt", "<Alt>" },
-
-        { "ctrl", "<Ctrl>" },
-        { "rctrl", "<Ctrl>" },
-        { "rightctrl", "<Ctrl>" },
-        { "right-ctrl", "<Ctrl>" },
-        { "lctrl", "<Ctrl>" },
-        { "leftctrl", "<Ctrl>" },
-        { "left-ctrl", "<Ctrl>" },
-
-        { "shift", "<Shift>" },
-        { "rshift", "<Shift>" },
-        { "rightshift", "<Shift>" },
-        { "right-shift", "<Shift>" },
-        { "lshift", "<Shift>" },
-        { "leftshift", "<Shift>" },
-        { "left-shift", "<Shift>" },
-
-        { "cmd", "<Ctrl>" },
-        { "rcmd", "<Ctrl>" },
-        { "rightcmd", "<Ctrl>" },
-        { "right-cmd", "<Ctrl>" },
-        { "lcmd", "<Ctrl>" },
-        { "leftcmd", "<Ctrl>" },
-        { "left-cmd", "<Ctrl>" },
-
-        { "win", "<Super>" },
-        { "rwin", "<Super>" },
-        { "rightwin", "<Super>" },
-        { "right-win", "<Super>" },
-        { "lwin", "<Super>" },
-        { "leftwin", "<Super>" },
-        { "left-win", "<Super>" },
-
-        { "esc", "Escape" },
-        /* { "escape", "Escape" }, */
-
-        { "ins", "Insert" },
-        /* { "insert", "Insert" }, */
-
-        { "del", "Delete" },
-        /* { "delete", "Delete" }, */
-
-        { "pgup", "Page_Up" },
-        { "pageup", "Page_Up" },
-        { "pgdn", "Page_Down" },
-        { "pagedown", "Page_Down" },
-
-        /* { "home", "home" }, */
-        /* { "end", "end" }, */
-        /* { "space", "space" }, */
-
-        { "enter", "Return" },
-
-        /* { "tab", "tab" }, */
-        /* { "f1", "F1" }, */
-        /* { "f2", "F2" }, */
-        /* { "f3", "F3" }, */
-        /* { "f4", "F4" }, */
-        /* { "f5", "F5" }, */
-        /* { "f6", "F6" }, */
-        /* { "f7", "F7" }, */
-        /* { "f8", "F8" }, */
-        /* { "f9", "F9" }, */
-        /* { "f10", "F10" }, */
-        /* { "f11", "F11" }, */
-        /* { "f12", "F12" } */
-    };
-
-    for (i = 0; i < G_N_ELEMENTS(keys); ++i) {
-        if (g_ascii_strcasecmp(keys[i].ctrl, key) == 0)
-            return g_strdup(keys[i].gtk);
-    }
-
-    return g_ascii_strup(key, -1);
-}
-
-static gchar*
-ctrl_key_to_gtk_accelerator(const gchar *key)
-{
-    gchar *accel, **k, **keyv;
-
-    keyv = g_strsplit(key, "+", -1);
-    g_return_val_if_fail(keyv != NULL, NULL);
-
-    for (k = keyv; *k != NULL; k++) {
-        gchar *tmp = *k;
-        *k = ctrl_key_to_gtk_key(tmp);
-        g_free(tmp);
-    }
-
-    accel = g_strjoinv(NULL, keyv);
-    g_strfreev(keyv);
-
-    return accel;
-}
-
 static void
 app_notified(VirtViewerApp *app,
              GParamSpec *pspec,
@@ -586,7 +485,9 @@ spice_ctrl_notified(SpiceCtrlController *ctrl,
         g_str_equal(pspec->name, "enable-smartcard") ||
         g_str_equal(pspec->name, "color-depth") ||
         g_str_equal(pspec->name, "disable-effects") ||
-        g_str_equal(pspec->name, "enable-usbredir")) {
+        g_str_equal(pspec->name, "enable-usbredir") ||
+        g_str_equal(pspec->name, "secure-channels") ||
+        g_str_equal(pspec->name, "proxy")) {
         g_object_set_property(G_OBJECT(session), pspec->name, &value);
     } else if (g_str_equal(pspec->name, "sport")) {
         g_object_set_property(G_OBJECT(session), "tls-port", &value);
@@ -609,7 +510,7 @@ spice_ctrl_notified(SpiceCtrlController *ctrl,
                                   &value);
         }
     } else if (g_str_equal(pspec->name, "title")) {
-        g_object_set_property(G_OBJECT(app), "title", &value);
+        virt_viewer_app_set_title(app, g_value_get_string(&value));
     } else if (g_str_equal(pspec->name, "display-flags")) {
         guint flags = g_value_get_uint(&value);
         gboolean fullscreen = flags & CONTROLLER_SET_FULL_SCREEN;
@@ -620,41 +521,7 @@ spice_ctrl_notified(SpiceCtrlController *ctrl,
     } else if (g_str_equal(pspec->name, "menu")) {
         spice_ctrl_menu_updated(self);
     } else if (g_str_equal(pspec->name, "hotkeys")) {
-        gchar **hotkey, **hotkeys = g_strsplit(g_value_get_string(&value), ",", -1);
-        if (!hotkeys || g_strv_length(hotkeys) == 0) {
-            g_object_set(app, "enable-accel", FALSE, NULL);
-            goto end;
-        }
-
-        for (hotkey = hotkeys; *hotkey != NULL; hotkey++) {
-            gchar *key = strstr(*hotkey, "=");
-            if (key == NULL) {
-                g_warn_if_reached();
-                continue;
-            }
-            *key = '\0';
-
-            gchar *accel = ctrl_key_to_gtk_accelerator(key + 1);
-            guint accel_key;
-            GdkModifierType accel_mods;
-            gtk_accelerator_parse(accel, &accel_key, &accel_mods);
-            g_free(accel);
-
-            if (g_str_equal(*hotkey, "toggle-fullscreen")) {
-                gtk_accel_map_change_entry("<virt-viewer>/view/fullscreen", accel_key, accel_mods, TRUE);
-            } else if (g_str_equal(*hotkey, "release-cursor")) {
-                gtk_accel_map_change_entry("<virt-viewer>/view/release-cursor", accel_key, accel_mods, TRUE);
-            } else if (g_str_equal(*hotkey, "smartcard-insert")) {
-                gtk_accel_map_change_entry("<virt-viewer>/file/smartcard-insert", accel_key, accel_mods, TRUE);
-            } else if (g_str_equal(*hotkey, "smartcard-remove")) {
-                gtk_accel_map_change_entry("<virt-viewer>/file/smartcard-remove", accel_key, accel_mods, TRUE);
-            } else {
-                g_warning("Unknown hotkey command %s", *hotkey);
-            }
-        }
-        g_strfreev(hotkeys);
-
-        g_object_set(app, "enable-accel", TRUE, NULL);
+        virt_viewer_app_set_hotkeys(app, g_value_get_string(&value));
     } else {
         gchar *content = g_strdup_value_contents(&value);
 
@@ -662,7 +529,6 @@ spice_ctrl_notified(SpiceCtrlController *ctrl,
         g_free(content);
     }
 
-end:
     g_object_unref(session);
     g_value_unset(&value);
 }
@@ -702,19 +568,19 @@ spice_ctrl_listen_async_cb(GObject *object,
 }
 
 
-static int
-remote_viewer_activate(VirtViewerApp *app)
+static gboolean
+remote_viewer_activate(VirtViewerApp *app, GError **error)
 {
-    g_return_val_if_fail(REMOTE_VIEWER_IS(app), -1);
+    g_return_val_if_fail(REMOTE_VIEWER_IS(app), FALSE);
     RemoteViewer *self = REMOTE_VIEWER(app);
-    int ret = -1;
+    gboolean ret = FALSE;
 
     if (self->priv->controller) {
         SpiceSession *session = remote_viewer_get_spice_session(self);
         ret = spice_session_connect(session);
         g_object_unref(session);
     } else {
-        ret = VIRT_VIEWER_APP_CLASS(remote_viewer_parent_class)->activate(app);
+        ret = VIRT_VIEWER_APP_CLASS(remote_viewer_parent_class)->activate(app, error);
     }
 
     return ret;
@@ -729,20 +595,230 @@ remote_viewer_window_added(VirtViewerApp *app G_GNUC_UNUSED,
 }
 #endif
 
+#ifdef HAVE_OVIRT
+static gboolean
+parse_ovirt_uri(const gchar *uri_str, char **rest_uri, char **name)
+{
+    char *vm_name = NULL;
+    char *rel_path;
+    xmlURIPtr uri;
+    gchar **path_elements;
+    guint element_count;
+
+    g_return_val_if_fail(uri_str != NULL, FALSE);
+    g_return_val_if_fail(rest_uri != NULL, FALSE);
+    g_return_val_if_fail(name != NULL, FALSE);
+
+    uri = xmlParseURI(uri_str);
+    if (uri == NULL)
+        return FALSE;
+
+    if (g_strcmp0(uri->scheme, "ovirt") != 0) {
+        xmlFreeURI(uri);
+        return FALSE;
+    }
+
+    if (uri->path == NULL) {
+        xmlFreeURI(uri);
+        return FALSE;
+    }
+
+    /* extract VM name from path */
+    path_elements = g_strsplit(uri->path, "/", -1);
+
+    element_count = g_strv_length(path_elements);
+    if (element_count == 0) {
+        g_strfreev(path_elements);
+        xmlFreeURI(uri);
+        return FALSE;
+    }
+    vm_name = path_elements[element_count-1];
+    path_elements[element_count-1] = NULL;
+
+    /* build final URI */
+    rel_path = g_strjoinv("/", path_elements);
+    /* FIXME: how to decide between http and https? */
+    *rest_uri = g_strdup_printf("https://%s/%s/api/", uri->server, rel_path);
+    *name = vm_name;
+    g_free(rel_path);
+    g_strfreev(path_elements);
+    xmlFreeURI(uri);
+
+    g_debug("oVirt base URI: %s", *rest_uri);
+    g_debug("oVirt VM name: %s", *name);
+
+    return TRUE;
+}
+
+static gboolean
+authenticate_cb(RestProxy *proxy, G_GNUC_UNUSED RestProxyAuth *auth,
+                G_GNUC_UNUSED gboolean retrying, gpointer user_data)
+{
+    gchar *username;
+    gchar *password;
+    VirtViewerWindow *window;
+
+    window = virt_viewer_app_get_main_window(VIRT_VIEWER_APP(user_data));
+    int ret = virt_viewer_auth_collect_credentials(virt_viewer_window_get_window(window),
+                                                   "oVirt",
+                                                   NULL,
+                                                   &username, &password);
+    if (ret < 0) {
+        return FALSE;
+    } else {
+        g_object_set(G_OBJECT(proxy),
+                     "username", username,
+                     "password", password,
+                     NULL);
+        g_free(username);
+        g_free(password);
+        return TRUE;
+    }
+}
+
+
+static gboolean
+create_ovirt_session(VirtViewerApp *app, const char *uri)
+{
+    OvirtProxy *proxy = NULL;
+    OvirtVm *vm = NULL;
+    OvirtVmDisplay *display = NULL;
+    OvirtVmState state;
+    GError *error = NULL;
+    char *rest_uri = NULL;
+    char *vm_name = NULL;
+    gboolean success = FALSE;
+    guint port;
+    guint secure_port;
+    OvirtVmDisplayType type;
+    const char *session_type;
+
+    gchar *gport = NULL;
+    gchar *gtlsport = NULL;
+    gchar *ghost = NULL;
+    gchar *ticket = NULL;
+
+    g_return_val_if_fail(VIRT_VIEWER_IS_APP(app), FALSE);
+
+    if (!parse_ovirt_uri(uri, &rest_uri, &vm_name))
+        goto error;
+    proxy = ovirt_proxy_new(rest_uri);
+    if (proxy == NULL)
+        goto error;
+    g_signal_connect(G_OBJECT(proxy), "authenticate",
+                     G_CALLBACK(authenticate_cb), app);
+
+    ovirt_proxy_fetch_ca_certificate(proxy, &error);
+    if (error != NULL) {
+        g_debug("failed to get CA certificate: %s", error->message);
+        goto error;
+    }
+
+    ovirt_proxy_fetch_vms(proxy, &error);
+    if (error != NULL) {
+        g_debug("failed to lookup %s: %s", vm_name, error->message);
+        goto error;
+    }
+
+    vm = ovirt_proxy_lookup_vm(proxy, vm_name);
+    g_return_val_if_fail(vm != NULL, FALSE);
+    g_object_get(G_OBJECT(vm), "state", &state, NULL);
+    if (state != OVIRT_VM_STATE_UP) {
+        g_debug("oVirt VM %s is not running", vm_name);
+        goto error;
+    }
+
+    if (!ovirt_vm_get_ticket(vm, proxy, &error)) {
+        g_debug("failed to get ticket for %s: %s", vm_name, error->message);
+        goto error;
+    }
+
+    g_object_get(G_OBJECT(vm), "display", &display, NULL);
+    if (display == NULL) {
+        goto error;
+    }
+
+    g_object_get(G_OBJECT(display),
+                 "type", &type,
+                 "address", &ghost,
+                 "port", &port,
+                 "secure-port", &secure_port,
+                 "ticket", &ticket,
+                 NULL);
+    gport = g_strdup_printf("%d", port);
+    gtlsport = g_strdup_printf("%d", secure_port);
+
+    if (type == OVIRT_VM_DISPLAY_SPICE) {
+        session_type = "spice";
+    } else if (type == OVIRT_VM_DISPLAY_VNC) {
+        session_type = "vnc";
+    } else {
+        g_debug("Unknown display type: %d", type);
+        goto error;
+    }
+
+    virt_viewer_app_set_connect_info(app, NULL, ghost, gport, gtlsport,
+                                     session_type, NULL, NULL, 0, NULL);
+
+    if (virt_viewer_app_create_session(app, session_type) < 0)
+        goto error;
+
+#ifdef HAVE_SPICE_GTK
+    if (type == OVIRT_VM_DISPLAY_SPICE) {
+        SpiceSession *session;
+        GByteArray *ca_cert;
+
+        g_object_get(G_OBJECT(proxy), "ca-cert", &ca_cert, NULL);
+        session = remote_viewer_get_spice_session(REMOTE_VIEWER(app));
+        g_object_set(G_OBJECT(session),
+                     "ca", ca_cert,
+                     "password", ticket,
+                     NULL);
+        g_byte_array_unref(ca_cert);
+    }
+#endif
+
+    success = TRUE;
+
+error:
+    g_free(rest_uri);
+    g_free(vm_name);
+    g_free(ticket);
+    g_free(gport);
+    g_free(gtlsport);
+    g_free(ghost);
+
+    if (error != NULL)
+        g_error_free(error);
+    if (display != NULL)
+        g_object_unref(display);
+    if (vm != NULL)
+        g_object_unref(vm);
+    if (proxy != NULL)
+        g_object_unref(proxy);
+
+    return success;
+}
+
+#endif
+
 static gboolean
 remote_viewer_start(VirtViewerApp *app)
 {
     g_return_val_if_fail(REMOTE_VIEWER_IS(app), FALSE);
 
-#if HAVE_SPICE_GTK
+#ifdef HAVE_SPICE_GTK
     RemoteViewer *self = REMOTE_VIEWER(app);
     RemoteViewerPrivate *priv = self->priv;
 #endif
+    GFile *file = NULL;
+    VirtViewerFile *vvfile = NULL;
     gboolean ret = FALSE;
     gchar *guri = NULL;
     gchar *type = NULL;
+    GError *error = NULL;
 
-#if HAVE_SPICE_GTK
+#ifdef HAVE_SPICE_GTK
     g_signal_connect(app, "notify", G_CALLBACK(app_notified), self);
 
     if (priv->controller) {
@@ -768,29 +844,59 @@ remote_viewer_start(VirtViewerApp *app)
         g_return_val_if_fail(guri != NULL, FALSE);
 
         DEBUG_LOG("Opening display to %s", guri);
-        g_object_set(app, "title", guri, NULL);
+        if (virt_viewer_app_get_title(app) == NULL)
+            virt_viewer_app_set_title(app, guri);
 
-        if (virt_viewer_util_extract_host(guri, &type, NULL, NULL, NULL, NULL) < 0 || type == NULL) {
+        file = g_file_new_for_commandline_arg(guri);
+        if (g_file_query_exists(file, NULL)) {
+            gchar *path = g_file_get_path(file);
+            vvfile = virt_viewer_file_new(path, &error);
+            g_free(path);
+            if (error) {
+                virt_viewer_app_simple_message_dialog(app, _("Invalid file %s"), guri);
+                g_warning("%s", error->message);
+                g_clear_error(&error);
+                goto cleanup;
+            }
+            g_object_get(G_OBJECT(vvfile), "type", &type, NULL);
+        } else if (virt_viewer_util_extract_host(guri, &type, NULL, NULL, NULL, NULL) < 0 || type == NULL) {
             virt_viewer_app_simple_message_dialog(app, _("Cannot determine the connection type from URI"));
             goto cleanup;
         }
-
-        if (virt_viewer_app_create_session(app, type) < 0) {
-            virt_viewer_app_simple_message_dialog(app, _("Couldn't create a session for this type: %s"), type);
-            goto cleanup;
+#ifdef HAVE_OVIRT
+        if (g_strcmp0(type, "ovirt") == 0) {
+            if (!create_ovirt_session(app, guri)) {
+                virt_viewer_app_simple_message_dialog(app, _("Couldn't open oVirt session"));
+                goto cleanup;
+            }
+        } else
+#endif
+        {
+            if (virt_viewer_app_create_session(app, type) < 0) {
+                virt_viewer_app_simple_message_dialog(app, _("Couldn't create a session for this type: %s"), type);
+                goto cleanup;
+            }
         }
 
-        if (virt_viewer_app_initial_connect(app) < 0) {
-            virt_viewer_app_simple_message_dialog(app, _("Failed to initiate connection"));
+        virt_viewer_session_set_file(virt_viewer_app_get_session(app), vvfile);
+
+        if (!virt_viewer_app_initial_connect(app, &error)) {
+            const gchar *msg = error ? error->message :
+                _("Failed to initiate connection");
+
+            virt_viewer_app_simple_message_dialog(app, msg);
+            g_clear_error(&error);
             goto cleanup;
         }
-#if HAVE_SPICE_GTK
+#ifdef HAVE_SPICE_GTK
     }
 #endif
 
     ret = VIRT_VIEWER_APP_CLASS(remote_viewer_parent_class)->start(app);
 
- cleanup:
+cleanup:
+    g_clear_object(&file);
+    g_clear_object(&vvfile);
     g_free(guri);
     g_free(type);
     return ret;
