@@ -140,6 +140,8 @@ struct _VirtViewerAppPrivate {
     char *title;
 
     gint focused;
+    GKeyFile *config;
+    gchar *config_file;
 };
 
 
@@ -177,7 +179,8 @@ virt_viewer_app_set_debug(gboolean debug)
         const gchar *doms = g_getenv("G_MESSAGES_DEBUG");
         if (!doms) {
             g_setenv("G_MESSAGES_DEBUG", G_LOG_DOMAIN, 1);
-        } else if (!strstr(doms, G_LOG_DOMAIN)) {
+        } else if (!g_str_equal(doms, "all") &&
+                   !strstr(doms, G_LOG_DOMAIN)) {
             gchar *newdoms = g_strdup_printf("%s %s", doms, G_LOG_DOMAIN);
             g_setenv("G_MESSAGES_DEBUG", newdoms, 1);
             g_free(newdoms);
@@ -218,12 +221,34 @@ virt_viewer_app_simple_message_dialog(VirtViewerApp *self,
     g_free(msg);
 }
 
+static void
+virt_viewer_app_save_config(VirtViewerApp *self)
+{
+    VirtViewerAppPrivate *priv = self->priv;
+    GError *error = NULL;
+    gchar *dir, *data;
+
+    dir = g_path_get_dirname(priv->config_file);
+    if (g_mkdir_with_parents(dir, S_IRWXU) == -1)
+        g_warning("failed to create config directory");
+    g_free(dir);
+
+    if ((data = g_key_file_to_data(priv->config, NULL, &error)) == NULL ||
+        !g_file_set_contents(priv->config_file, data, -1, &error)) {
+        g_warning("Couldn't save configuration: %s", error->message);
+        g_clear_error(&error);
+    }
+    g_free(data);
+}
+
 void
 virt_viewer_app_quit(VirtViewerApp *self)
 {
     g_return_if_fail(VIRT_VIEWER_IS_APP(self));
     VirtViewerAppPrivate *priv = self->priv;
 
+    virt_viewer_app_save_config(self);
+
     if (priv->session) {
         virt_viewer_session_close(VIRT_VIEWER_SESSION(priv->session));
         if (priv->connected) {
@@ -235,6 +260,52 @@ virt_viewer_app_quit(VirtViewerApp *self)
     gtk_main_quit();
 }
 
+void
+virt_viewer_app_maybe_quit(VirtViewerApp *self, VirtViewerWindow *window)
+{
+    GError *error = NULL;
+
+    gboolean ask = g_key_file_get_boolean(self->priv->config,
+                                          "virt-viewer", "ask-quit", &error);
+    if (error) {
+        ask = TRUE;
+        g_clear_error(&error);
+    }
+
+    if (ask) {
+        GtkWidget *dialog =
+            gtk_message_dialog_new (virt_viewer_window_get_window(window),
+                    GTK_DIALOG_DESTROY_WITH_PARENT,
+                    GTK_MESSAGE_QUESTION,
+                    GTK_BUTTONS_OK_CANCEL,
+                    _("Do you want to close the session?"));
+
+        GtkWidget *check = gtk_check_button_new_with_label(_("Do not ask me again"));
+        gtk_container_add(GTK_CONTAINER(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), check);
+        gtk_widget_show(check);
+
+        gint result = gtk_dialog_run(GTK_DIALOG(dialog));
+
+        gboolean dont_ask = FALSE;
+        g_object_get(check, "active", &dont_ask, NULL);
+        if (dont_ask)
+            g_key_file_set_boolean(self->priv->config,
+                    "virt-viewer", "ask-quit", FALSE);
+
+        gtk_widget_destroy(dialog);
+        switch (result) {
+            case GTK_RESPONSE_OK:
+                virt_viewer_app_quit(self);
+                break;
+            default:
+                break;
+        }
+    } else {
+        virt_viewer_app_quit(self);
+    }
+
+}
+
 static void count_window_visible(gpointer key G_GNUC_UNUSED,
                                  gpointer value,
                                  gpointer user_data)
@@ -254,12 +325,6 @@ virt_viewer_app_get_n_windows_visible(VirtViewerApp *self)
     return n;
 }
 
-static guint
-virt_viewer_app_get_n_windows(VirtViewerApp *self)
-{
-    return g_hash_table_size(self->priv->windows);
-}
-
 gboolean
 virt_viewer_app_window_set_visible(VirtViewerApp *self,
                                    VirtViewerWindow *window,
@@ -275,27 +340,9 @@ virt_viewer_app_window_set_visible(VirtViewerApp *self,
         if (virt_viewer_app_get_n_windows_visible(self) > 1) {
             virt_viewer_window_hide(window);
             return FALSE;
-        } else if (virt_viewer_app_get_n_windows(self) > 1) {
-            GtkWidget *dialog =
-                gtk_message_dialog_new (virt_viewer_window_get_window(window),
-                                        GTK_DIALOG_DESTROY_WITH_PARENT,
-                                        GTK_MESSAGE_QUESTION,
-                                        GTK_BUTTONS_OK_CANCEL,
-                                        _("This is the last visible display. Do you want to quit?"));
-            gint result = gtk_dialog_run (GTK_DIALOG (dialog));
-            gtk_widget_destroy(dialog);
-            switch (result) {
-            case GTK_RESPONSE_OK:
-                virt_viewer_app_quit(self);
-                break;
-            default:
-                break;
-            }
-            return FALSE;
-        } else {
-            virt_viewer_app_quit(self);
-            return FALSE;
         }
+
+        virt_viewer_app_maybe_quit(self, window);
     }
 
     g_warn_if_reached();
@@ -431,6 +478,7 @@ virt_viewer_app_trace(VirtViewerApp *self,
         va_start(ap, fmt);
         g_vprintf(fmt, ap);
         va_end(ap);
+        g_print("\n");
     }
 }
 
@@ -542,6 +590,10 @@ virt_viewer_app_set_nth_window(VirtViewerApp *self, gint nth, VirtViewerWindow *
     g_hash_table_insert(self->priv->windows, key, win);
     virt_viewer_app_set_window_subtitle(self, win, nth);
     virt_viewer_app_update_menu_displays(self);
+    if (self->priv->session) {
+        gboolean has_usb = virt_viewer_session_has_usb(self->priv->session);
+        virt_viewer_window_set_usb_options_sensitive(win, has_usb);
+    }
 
     g_signal_emit(self, signals[SIGNAL_WINDOW_ADDED], 0, win);
 }
@@ -627,13 +679,13 @@ display_show_hint(VirtViewerDisplay *display,
     } else if (hint & VIRT_VIEWER_DISPLAY_SHOW_HINT_READY) {
         virt_viewer_notebook_show_display(nb);
         virt_viewer_window_show(win);
-        gtk_window_present(virt_viewer_window_get_window(win));
     } else {
         if (win != self->priv->main_window &&
             g_getenv("VIRT_VIEWER_HIDE"))
             virt_viewer_window_hide(win);
         virt_viewer_notebook_show_status(nb, _("Waiting for display %d..."), nth + 1);
     }
+    virt_viewer_app_update_menu_displays(self);
 
     g_object_unref(self);
 }
@@ -703,21 +755,21 @@ virt_viewer_app_create_session(VirtViewerApp *self, const gchar *type)
 #ifdef HAVE_GTK_VNC
     if (g_ascii_strcasecmp(type, "vnc") == 0) {
         GtkWindow *window = virt_viewer_window_get_window(priv->main_window);
-        virt_viewer_app_trace(self, "Guest %s has a %s display\n",
+        virt_viewer_app_trace(self, "Guest %s has a %s display",
                               priv->guest_name, type);
-        priv->session = virt_viewer_session_vnc_new(window);
+        priv->session = virt_viewer_session_vnc_new(self, window);
     } else
 #endif
 #ifdef HAVE_SPICE_GTK
     if (g_ascii_strcasecmp(type, "spice") == 0) {
         GtkWindow *window = virt_viewer_window_get_window(priv->main_window);
-        virt_viewer_app_trace(self, "Guest %s has a %s display\n",
+        virt_viewer_app_trace(self, "Guest %s has a %s display",
                               priv->guest_name, type);
         priv->session = virt_viewer_session_spice_new(self, window);
     } else
 #endif
     {
-        virt_viewer_app_trace(self, "Guest %s has unsupported %s display type\n",
+        virt_viewer_app_trace(self, "Guest %s has unsupported %s display type",
                               priv->guest_name, type);
         virt_viewer_app_simple_message_dialog(self, _("Unknown graphic type for the guest %s"),
                                               priv->guest_name);
@@ -814,14 +866,14 @@ virt_viewer_app_channel_open(VirtViewerSession *session G_GNUC_UNUSED,
 }
 #endif
 
-static int
-virt_viewer_app_default_activate(VirtViewerApp *self)
+static gboolean
+virt_viewer_app_default_activate(VirtViewerApp *self, GError **error)
 {
     VirtViewerAppPrivate *priv = self->priv;
     int fd = -1;
 
     if (!virt_viewer_app_open_connection(self, &fd))
-        return -1;
+        return FALSE;
 
     DEBUG_LOG("After open connection callback fd=%d", fd);
 
@@ -833,16 +885,16 @@ virt_viewer_app_default_activate(VirtViewerApp *self)
         gchar *p = NULL;
 
         if (priv->gport) {
-            virt_viewer_app_trace(self, "Opening indirect TCP connection to display at %s:%s\n",
+            virt_viewer_app_trace(self, "Opening indirect TCP connection to display at %s:%s",
                                   priv->ghost, priv->gport);
         } else {
-            virt_viewer_app_trace(self, "Opening indirect UNIX connection to display at %s\n",
+            virt_viewer_app_trace(self, "Opening indirect UNIX connection to display at %s",
                                   priv->unixsock);
         }
         if (priv->port)
             p = g_strdup_printf(":%d", priv->port);
 
-        virt_viewer_app_trace(self, "Setting up SSH tunnel via %s%s%s%s\n",
+        virt_viewer_app_trace(self, "Setting up SSH tunnel via %s%s%s%s",
                               priv->user ? priv->user : "",
                               priv->user ? "@" : "",
                               priv->host, p ? p : "");
@@ -851,53 +903,55 @@ virt_viewer_app_default_activate(VirtViewerApp *self)
         if ((fd = virt_viewer_app_open_tunnel_ssh(priv->host, priv->port,
                                                   priv->user, priv->ghost,
                                                   priv->gport, priv->unixsock)) < 0)
-            return -1;
+            return FALSE;
     } else if (priv->unixsock && fd == -1) {
         virt_viewer_app_trace(self, "Opening direct UNIX connection to display at %s",
                               priv->unixsock);
         if ((fd = virt_viewer_app_open_unix_sock(priv->unixsock)) < 0)
-            return -1;
+            return FALSE;
     }
 #endif
 
     if (fd >= 0) {
         return virt_viewer_session_open_fd(VIRT_VIEWER_SESSION(priv->session), fd);
     } else if (priv->guri) {
-        virt_viewer_app_trace(self, "Opening connection to display at %s\n", priv->guri);
-        return virt_viewer_session_open_uri(VIRT_VIEWER_SESSION(priv->session), priv->guri);
+        virt_viewer_app_trace(self, "Opening connection to display at %s", priv->guri);
+        return virt_viewer_session_open_uri(VIRT_VIEWER_SESSION(priv->session), priv->guri, error);
     } else {
-        virt_viewer_app_trace(self, "Opening direct TCP connection to display at %s:%s:%s\n",
+        virt_viewer_app_trace(self, "Opening direct TCP connection to display at %s:%s:%s",
                               priv->ghost, priv->gport, priv->gtlsport ? priv->gtlsport : "-1");
         return virt_viewer_session_open_host(VIRT_VIEWER_SESSION(priv->session),
                                              priv->ghost, priv->gport, priv->gtlsport);
     }
 
-    return -1;
+    return FALSE;
 }
 
-int
-virt_viewer_app_activate(VirtViewerApp *self)
+gboolean
+virt_viewer_app_activate(VirtViewerApp *self, GError **error)
 {
     VirtViewerAppPrivate *priv;
-    int ret;
+    gboolean ret;
 
-    g_return_val_if_fail(VIRT_VIEWER_IS_APP(self), -1);
+    g_return_val_if_fail(VIRT_VIEWER_IS_APP(self), FALSE);
 
     priv = self->priv;
     if (priv->active)
-        return -1;
+        return FALSE;
 
-    ret = VIRT_VIEWER_APP_GET_CLASS(self)->activate(self);
+    ret = VIRT_VIEWER_APP_GET_CLASS(self)->activate(self, error);
 
-    if (ret != -1) {
-        virt_viewer_app_show_status(self, _("Connecting to graphic server"));
+    if (ret == FALSE) {
         priv->connected = FALSE;
+    } else {
+        virt_viewer_app_show_status(self, _("Connecting to graphic server"));
         priv->cancelled = FALSE;
         priv->active = TRUE;
-        priv->grabbed = FALSE;
-        virt_viewer_app_update_title(self);
     }
 
+    priv->grabbed = FALSE;
+    virt_viewer_app_update_title(self);
+
     return ret;
 }
 
@@ -956,21 +1010,21 @@ static void virt_viewer_app_bell(VirtViewerSession *session G_GNUC_UNUSED,
 }
 
 
-static int
-virt_viewer_app_default_initial_connect(VirtViewerApp *self)
+static gboolean
+virt_viewer_app_default_initial_connect(VirtViewerApp *self, GError **error)
 {
-    return virt_viewer_app_activate(self);
+    return virt_viewer_app_activate(self, error);
 }
 
-int
-virt_viewer_app_initial_connect(VirtViewerApp *self)
+gboolean
+virt_viewer_app_initial_connect(VirtViewerApp *self, GError **error)
 {
     VirtViewerAppClass *klass;
 
-    g_return_val_if_fail(VIRT_VIEWER_IS_APP(self), -1);
+    g_return_val_if_fail(VIRT_VIEWER_IS_APP(self), FALSE);
     klass = VIRT_VIEWER_APP_GET_CLASS(self);
 
-    return klass->initial_connect(self);
+    return klass->initial_connect(self, error);
 }
 
 static gboolean
@@ -978,7 +1032,7 @@ virt_viewer_app_retryauth(gpointer opaque)
 {
     VirtViewerApp *self = opaque;
 
-    virt_viewer_app_initial_connect(self);
+    virt_viewer_app_initial_connect(self, NULL);
 
     return FALSE;
 }
@@ -992,7 +1046,7 @@ virt_viewer_app_connect_timer(void *opaque)
     DEBUG_LOG("Connect timer fired");
 
     if (!priv->active &&
-        virt_viewer_app_initial_connect(self) < 0)
+        virt_viewer_app_initial_connect(self, NULL) < 0)
         gtk_main_quit();
 
     if (priv->active) {
@@ -1009,6 +1063,8 @@ virt_viewer_app_start_reconnect_poll(VirtViewerApp *self)
     g_return_if_fail(VIRT_VIEWER_IS_APP(self));
     VirtViewerAppPrivate *priv = self->priv;
 
+    DEBUG_LOG("reconnect_poll: %d", priv->reconnect_poll);
+
     if (priv->reconnect_poll != 0)
         return;
 
@@ -1202,7 +1258,7 @@ virt_viewer_app_get_property (GObject *object, guint property_id,
         break;
 
     case PROP_FULLSCREEN_AUTO_CONF:
-        g_value_set_boolean(value, priv->fullscreen_auto_conf);
+        g_value_set_boolean(value, virt_viewer_app_get_fullscreen_auto_conf(self));
         break;
 
     default:
@@ -1244,9 +1300,7 @@ virt_viewer_app_set_property (GObject *object, guint property_id,
         break;
 
     case PROP_TITLE:
-        g_free(priv->title);
-        priv->title = g_value_dup_string(value);
-        virt_viewer_app_set_all_window_subtitles(self);
+        virt_viewer_app_set_title(self, g_value_get_string(value));
         break;
 
     case PROP_ENABLE_ACCEL:
@@ -1297,6 +1351,9 @@ virt_viewer_app_dispose (GObject *object)
     priv->guri = NULL;
     g_free(priv->title);
     priv->title = NULL;
+    g_free(priv->config_file);
+    priv->config_file = NULL;
+    g_clear_pointer(&priv->config, g_key_file_free);
 
     virt_viewer_app_free_connect_info(self);
 
@@ -1326,8 +1383,20 @@ gboolean virt_viewer_app_start(VirtViewerApp *self)
 static void
 virt_viewer_app_init (VirtViewerApp *self)
 {
+    GError *error = NULL;
+
     self->priv = GET_PRIVATE(self);
     self->priv->windows = g_hash_table_new_full(g_int_hash, g_int_equal, g_free, g_object_unref);
+    self->priv->config = g_key_file_new();
+    self->priv->config_file = g_build_filename(g_get_user_config_dir(),
+                                               "virt-viewer", "settings", NULL);
+
+    g_key_file_load_from_file(self->priv->config, self->priv->config_file,
+                    G_KEY_FILE_KEEP_COMMENTS|G_KEY_FILE_KEEP_TRANSLATIONS, &error);
+    if (error)
+        g_debug("Couldn't load configuration: %s", error->message);
+
+    g_clear_error(&error);
 }
 
 static GObject *
@@ -1493,6 +1562,22 @@ virt_viewer_app_class_init (VirtViewerAppClass *klass)
                      G_TYPE_OBJECT);
 }
 
+const char *virt_viewer_app_get_title(VirtViewerApp *self)
+{
+    g_return_val_if_fail(VIRT_VIEWER_IS_APP(self), NULL);
+
+    return self->priv->title;
+}
+
+void virt_viewer_app_set_title(VirtViewerApp *self, const char *title)
+{
+    g_return_if_fail(VIRT_VIEWER_IS_APP(self));
+
+    g_free(self->priv->title);
+    self->priv->title = g_strdup(title);
+    virt_viewer_app_set_all_window_subtitles(self);
+}
+
 void
 virt_viewer_app_set_direct(VirtViewerApp *self, gboolean direct)
 {
@@ -1501,6 +1586,59 @@ virt_viewer_app_set_direct(VirtViewerApp *self, gboolean direct)
     self->priv->direct = direct;
 }
 
+void
+virt_viewer_app_set_hotkeys(VirtViewerApp *self, const gchar *hotkeys_str)
+{
+    gchar **hotkey, **hotkeys = NULL;
+
+    g_return_if_fail(VIRT_VIEWER_IS_APP(self));
+
+    if (hotkeys_str)
+        hotkeys = g_strsplit(hotkeys_str, ",", -1);
+
+    if (!hotkeys || g_strv_length(hotkeys) == 0) {
+        g_strfreev(hotkeys);
+        g_object_set(self, "enable-accel", FALSE, NULL);
+        return;
+    }
+
+    /* Disable default bindings and replace them with our own */
+    gtk_accel_map_change_entry("<virt-viewer>/view/fullscreen", 0, 0, TRUE);
+    gtk_accel_map_change_entry("<virt-viewer>/view/release-cursor", 0, 0, TRUE);
+    gtk_accel_map_change_entry("<virt-viewer>/file/smartcard-insert", 0, 0, TRUE);
+    gtk_accel_map_change_entry("<virt-viewer>/file/smartcard-remove", 0, 0, TRUE);
+
+    for (hotkey = hotkeys; *hotkey != NULL; hotkey++) {
+        gchar *key = strstr(*hotkey, "=");
+        if (key == NULL) {
+            g_warn_if_reached();
+            continue;
+        }
+        *key = '\0';
+
+        gchar *accel = spice_hotkey_to_gtk_accelerator(key + 1);
+        guint accel_key;
+        GdkModifierType accel_mods;
+        gtk_accelerator_parse(accel, &accel_key, &accel_mods);
+        g_free(accel);
+
+        if (g_str_equal(*hotkey, "toggle-fullscreen")) {
+            gtk_accel_map_change_entry("<virt-viewer>/view/fullscreen", accel_key, accel_mods, TRUE);
+        } else if (g_str_equal(*hotkey, "release-cursor")) {
+            gtk_accel_map_change_entry("<virt-viewer>/view/release-cursor", accel_key, accel_mods, TRUE);
+        } else if (g_str_equal(*hotkey, "smartcard-insert")) {
+            gtk_accel_map_change_entry("<virt-viewer>/file/smartcard-insert", accel_key, accel_mods, TRUE);
+        } else if (g_str_equal(*hotkey, "smartcard-remove")) {
+            gtk_accel_map_change_entry("<virt-viewer>/file/smartcard-remove", accel_key, accel_mods, TRUE);
+        } else {
+            g_warning("Unknown hotkey command %s", *hotkey);
+        }
+    }
+    g_strfreev(hotkeys);
+
+    g_object_set(self, "enable-accel", TRUE, NULL);
+}
+
 void
 virt_viewer_app_set_attach(VirtViewerApp *self, gboolean attach)
 {
@@ -1540,11 +1678,12 @@ virt_viewer_app_update_pretty_address(VirtViewerApp *self)
 
     priv = self->priv;
     g_free(priv->pretty_address);
+    priv->pretty_address = NULL;
     if (priv->guri)
         priv->pretty_address = g_strdup(priv->guri);
     else if (priv->gport)
         priv->pretty_address = g_strdup_printf("%s:%s", priv->ghost, priv->gport);
-    else
+    else if (priv->host && priv->unixsock)
         priv->pretty_address = g_strdup_printf("%s:%s", priv->host, priv->unixsock);
 }
 
@@ -1564,18 +1703,32 @@ static void fullscreen_cb(gpointer key,
     DEBUG_LOG("fullscreen display %d: %d", nth, options->fullscreen);
     if (options->fullscreen) {
         GdkScreen *screen = gdk_screen_get_default();
-        GdkRectangle mon;
 
         if (nth >= gdk_screen_get_n_monitors(screen)) {
             DEBUG_LOG("skipping display %d", nth);
             return;
         }
-        gdk_screen_get_monitor_geometry(screen, nth, &mon);
-        virt_viewer_window_enter_fullscreen(vwin, options->move, mon.x, mon.y);
+        virt_viewer_window_enter_fullscreen(vwin, options->move ? nth : -1);
     } else
         virt_viewer_window_leave_fullscreen(vwin);
 }
 
+gboolean
+virt_viewer_app_get_fullscreen(VirtViewerApp *self)
+{
+    g_return_val_if_fail(VIRT_VIEWER_IS_APP(self), FALSE);
+
+    return self->priv->fullscreen;
+}
+
+gboolean
+virt_viewer_app_get_fullscreen_auto_conf(VirtViewerApp *self)
+{
+    g_return_val_if_fail(VIRT_VIEWER_IS_APP(self), FALSE);
+
+    return self->priv->fullscreen_auto_conf;
+}
+
 static void
 virt_viewer_app_set_fullscreen(VirtViewerApp *self, gboolean fullscreen)
 {
@@ -1626,19 +1779,48 @@ update_menu_displays_sort(gconstpointer a, gconstpointer b)
         return 0;
 }
 
+static GtkMenuShell *
+window_empty_display_submenu(VirtViewerWindow *window)
+{
+    /* Because of what apparently is a gtk+2 bug (rhbz#922712), we
+     * cannot recreate the submenu every time we need to refresh it,
+     * otherwise the application may get frozen with the keyboard and
+     * mouse grabbed if gtk_menu_item_set_submenu is called while
+     * the menu is displayed. Reusing the same menu every time
+     * works around this issue.
+     */
+    GtkMenuItem *menu = virt_viewer_window_get_menu_displays(window);
+    GtkMenuShell *submenu;
+
+    submenu = GTK_MENU_SHELL(gtk_menu_item_get_submenu(menu));
+    if (submenu) {
+        GList *subitems;
+        GList *it;
+        subitems = gtk_container_get_children(GTK_CONTAINER(submenu));
+        for (it = subitems; it != NULL; it = it->next) {
+            gtk_container_remove(GTK_CONTAINER(submenu), GTK_WIDGET(it->data));
+        }
+        g_list_free(subitems);
+    } else {
+        submenu = GTK_MENU_SHELL(gtk_menu_new());
+        gtk_menu_item_set_submenu(menu, GTK_WIDGET(submenu));
+    }
+
+    return submenu;
+}
+
 static void
 window_update_menu_displays_cb(gpointer key G_GNUC_UNUSED,
                                gpointer value,
                                gpointer user_data)
 {
     VirtViewerApp *self = VIRT_VIEWER_APP(user_data);
-    VirtViewerWindow *window = VIRT_VIEWER_WINDOW(value);
-    GtkMenuShell *submenu = GTK_MENU_SHELL(gtk_menu_new());
-    GtkMenuItem *menu = virt_viewer_window_get_menu_displays(window);
+    GtkMenuShell *submenu;
     GList *keys = g_hash_table_get_keys(self->priv->windows);
     GList *tmp;
 
     keys = g_list_sort(keys, update_menu_displays_sort);
+    submenu = window_empty_display_submenu(VIRT_VIEWER_WINDOW(value));
 
     tmp = keys;
     while (tmp) {
@@ -1646,7 +1828,7 @@ window_update_menu_displays_cb(gpointer key G_GNUC_UNUSED,
         VirtViewerWindow *vwin = VIRT_VIEWER_WINDOW(g_hash_table_lookup(self->priv->windows, nth));
         VirtViewerDisplay *display = virt_viewer_window_get_display(vwin);
         GtkWidget *item;
-        gboolean visible, sensitive = FALSE;
+        gboolean visible, sensitive;
         gchar *label;
 
         label = g_strdup_printf(_("Display %d"), *nth + 1);
@@ -1656,17 +1838,16 @@ window_update_menu_displays_cb(gpointer key G_GNUC_UNUSED,
         visible = gtk_widget_get_visible(GTK_WIDGET(virt_viewer_window_get_window(vwin)));
         gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), visible);
 
+        sensitive = visible;
         if (display) {
             guint hint = virt_viewer_display_get_show_hint(display);
 
             if (hint & VIRT_VIEWER_DISPLAY_SHOW_HINT_READY)
                 sensitive = TRUE;
 
-            if ((hint & VIRT_VIEWER_DISPLAY_SHOW_HINT_DISABLED) &&
-                virt_viewer_display_get_selectable(display))
+            if (virt_viewer_display_get_selectable(display))
                 sensitive = TRUE;
         }
-
         gtk_widget_set_sensitive(item, sensitive);
 
         g_signal_connect(G_OBJECT(item),
@@ -1676,7 +1857,6 @@ window_update_menu_displays_cb(gpointer key G_GNUC_UNUSED,
     }
 
     gtk_widget_show_all(GTK_WIDGET(submenu));
-    gtk_menu_item_set_submenu(menu, GTK_WIDGET(submenu));
     g_list_free(keys);
 }
 
@@ -1718,7 +1898,7 @@ virt_viewer_app_set_connect_info(VirtViewerApp *self,
     priv->host = g_strdup(host);
     priv->ghost = g_strdup(ghost);
     priv->gport = g_strdup(gport);
-    priv->gtlsport = gtlsport ? g_strdup(gtlsport) : NULL;
+    priv->gtlsport = g_strdup(gtlsport);
     priv->transport = g_strdup(transport);
     priv->unixsock = g_strdup(unixsock);
     priv->user = g_strdup(user);
