@@ -24,6 +24,7 @@
 
 #include <config.h>
 
+#include <math.h>
 #include <spice-audio.h>
 
 #include <glib/gi18n.h>
@@ -37,6 +38,13 @@ G_DEFINE_TYPE (VirtViewerDisplaySpice, virt_viewer_display_spice, VIRT_VIEWER_TY
 struct _VirtViewerDisplaySpicePrivate {
     SpiceChannel *channel; /* weak reference */
     SpiceDisplay *display;
+    int auto_resize;
+};
+
+enum {
+    AUTO_RESIZE_ALWAYS,
+    AUTO_RESIZE_FULLSCREEN,
+    AUTO_RESIZE_NEVER,
 };
 
 #define VIRT_VIEWER_DISPLAY_SPICE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE((o), VIRT_VIEWER_TYPE_DISPLAY_SPICE, VirtViewerDisplaySpicePrivate))
@@ -91,14 +99,15 @@ show_hint_changed(VirtViewerDisplay *self)
 {
     SpiceMainChannel *main_channel = get_main(self);
     guint enabled = TRUE;
-    guint nth;
+    guint nth, hint = virt_viewer_display_get_show_hint(self);
 
     /* this may happen when finalizing */
     if (!main_channel)
         return;
 
     g_object_get(self, "nth-display", &nth, NULL);
-    if (virt_viewer_display_get_show_hint(self) & VIRT_VIEWER_DISPLAY_SHOW_HINT_DISABLED)
+    if (!(hint & VIRT_VIEWER_DISPLAY_SHOW_HINT_SET) ||
+        hint & VIRT_VIEWER_DISPLAY_SHOW_HINT_DISABLED)
         enabled = FALSE;
 
     spice_main_set_display_enabled(main_channel, nth, enabled);
@@ -181,6 +190,8 @@ virt_viewer_display_spice_size_allocate(VirtViewerDisplaySpice *self,
     gdouble dw = allocation->width, dh = allocation->height;
     guint zoom = 100;
     guint nth;
+    gint x = 0, y = 0;
+    gboolean disable_display_position = TRUE;
 
     if (virt_viewer_display_get_auto_resize(VIRT_VIEWER_DISPLAY(self)) == FALSE)
         return;
@@ -188,17 +199,47 @@ virt_viewer_display_spice_size_allocate(VirtViewerDisplaySpice *self,
     if (virt_viewer_display_get_show_hint(VIRT_VIEWER_DISPLAY(self)) & VIRT_VIEWER_DISPLAY_SHOW_HINT_DISABLED)
         return;
 
+    if (self->priv->auto_resize == AUTO_RESIZE_FULLSCREEN) {
+        GdkRectangle monitor;
+        GdkScreen *screen = gtk_widget_get_screen(GTK_WIDGET(self));
+        int n = virt_viewer_display_get_monitor(VIRT_VIEWER_DISPLAY(self));
+        if (n == -1)
+            n = gdk_screen_get_monitor_at_window(screen,
+                                     gtk_widget_get_window(GTK_WIDGET(self)));
+        gdk_screen_get_monitor_geometry(screen, n, &monitor);
+        disable_display_position = FALSE;
+        x = monitor.x;
+        y = monitor.y;
+        dw = monitor.width;
+        dh = monitor.height;
+    } else {
+        GtkWidget *top = gtk_widget_get_toplevel(GTK_WIDGET(self));
+        gtk_window_get_position(GTK_WINDOW(top), &x, &y);
+        if (x < 0)
+            x = 0;
+        if (y < 0)
+            y = 0;
+    }
+
     if (virt_viewer_display_get_zoom(VIRT_VIEWER_DISPLAY(self))) {
         zoom = virt_viewer_display_get_zoom_level(VIRT_VIEWER_DISPLAY(self));
 
-        dw /= ((double)zoom / 100.0);
-        dh /= ((double)zoom / 100.0);
+        dw = round(dw * 100 / zoom);
+        dh = round(dh * 100 / zoom);
     }
 
     g_object_get(self, "nth-display", &nth, NULL);
 
-    spice_main_set_display(get_main(VIRT_VIEWER_DISPLAY(self)),
-                           nth, 0, 0, dw, dh);
+    if (self->priv->auto_resize != AUTO_RESIZE_NEVER) {
+        g_object_set(get_main(VIRT_VIEWER_DISPLAY(self)),
+                     "disable-display-position", disable_display_position,
+                     "disable-display-align", !disable_display_position,
+                     NULL);
+        spice_main_set_display(get_main(VIRT_VIEWER_DISPLAY(self)),
+                               nth, x, y, dw, dh);
+    }
+    if (self->priv->auto_resize == AUTO_RESIZE_FULLSCREEN)
+        self->priv->auto_resize = AUTO_RESIZE_NEVER;
 }
 
 static void
@@ -206,15 +247,33 @@ enable_accel_changed(VirtViewerApp *app,
                      GParamSpec *pspec G_GNUC_UNUSED,
                      VirtViewerDisplaySpice *self)
 {
-    if (virt_viewer_app_get_enable_accel(app)) {
+    if (virt_viewer_app_get_enable_accel(app)
+            && gtk_accel_map_lookup_entry("<virt-viewer>/view/release-cursor", NULL)) {
+        SpiceGrabSequence *seq = spice_grab_sequence_new(0, NULL);
         /* disable default grab sequence */
-        spice_display_set_grab_keys(self->priv->display,
-                                    spice_grab_sequence_new(0, NULL));
+        spice_display_set_grab_keys(self->priv->display, seq);
+        spice_grab_sequence_free(seq);
     } else {
         spice_display_set_grab_keys(self->priv->display, NULL);
     }
 }
 
+static void
+fullscreen_changed(VirtViewerApp *app,
+                   GParamSpec *pspec G_GNUC_UNUSED,
+                   VirtViewerDisplaySpice *self)
+{
+    if (virt_viewer_app_get_fullscreen(app)) {
+        gboolean auto_conf;
+        g_object_get(app, "fullscreen-auto-conf", &auto_conf, NULL);
+        if (auto_conf)
+            self->priv->auto_resize = AUTO_RESIZE_NEVER;
+        else
+            self->priv->auto_resize = AUTO_RESIZE_FULLSCREEN;
+    } else
+        self->priv->auto_resize = AUTO_RESIZE_ALWAYS;
+}
+
 GtkWidget *
 virt_viewer_display_spice_new(VirtViewerSessionSpice *session,
                               SpiceChannel *channel,
@@ -267,6 +326,9 @@ virt_viewer_display_spice_new(VirtViewerSessionSpice *session,
     app = virt_viewer_session_get_app(VIRT_VIEWER_SESSION(session));
     virt_viewer_signal_connect_object(app, "notify::enable-accel",
                                       G_CALLBACK(enable_accel_changed), self, 0);
+    virt_viewer_signal_connect_object(app, "notify::fullscreen",
+                                      G_CALLBACK(fullscreen_changed), self, 0);
+    fullscreen_changed(app, NULL, self);
     enable_accel_changed(app, NULL, self);
 
     return GTK_WIDGET(self);
