@@ -29,6 +29,7 @@
 
 #include <spice-option.h>
 #include <usb-device-widget.h>
+#include "virt-viewer-file.h"
 #include "virt-viewer-util.h"
 #include "virt-viewer-session-spice.h"
 #include "virt-viewer-display-spice.h"
@@ -63,7 +64,7 @@ enum {
 static void virt_viewer_session_spice_close(VirtViewerSession *session);
 static gboolean virt_viewer_session_spice_open_fd(VirtViewerSession *session, int fd);
 static gboolean virt_viewer_session_spice_open_host(VirtViewerSession *session, const gchar *host, const gchar *port, const gchar *tlsport);
-static gboolean virt_viewer_session_spice_open_uri(VirtViewerSession *session, const gchar *uri);
+static gboolean virt_viewer_session_spice_open_uri(VirtViewerSession *session, const gchar *uri, GError **error);
 static gboolean virt_viewer_session_spice_channel_open_fd(VirtViewerSession *session, VirtViewerSessionChannel *channel, int fd);
 static gboolean virt_viewer_session_spice_has_usb(VirtViewerSession *session);
 static void virt_viewer_session_spice_usb_device_selection(VirtViewerSession *session, GtkWindow *parent);
@@ -123,6 +124,12 @@ virt_viewer_session_spice_dispose(GObject *obj)
 }
 
 
+static const gchar*
+virt_viewer_session_spice_mime_type(VirtViewerSession *self G_GNUC_UNUSED)
+{
+    return "application/x-spice";
+}
+
 static void
 virt_viewer_session_spice_class_init(VirtViewerSessionSpiceClass *klass)
 {
@@ -142,6 +149,7 @@ virt_viewer_session_spice_class_init(VirtViewerSessionSpiceClass *klass)
     dclass->usb_device_selection = virt_viewer_session_spice_usb_device_selection;
     dclass->smartcard_insert = virt_viewer_session_spice_smartcard_insert;
     dclass->smartcard_remove = virt_viewer_session_spice_smartcard_remove;
+    dclass->mime_type = virt_viewer_session_spice_mime_type;
 
     g_type_class_add_private(klass, sizeof(VirtViewerSessionSpicePrivate));
 
@@ -245,16 +253,117 @@ virt_viewer_session_spice_open_host(VirtViewerSession *session,
     return spice_session_connect(self->priv->session);
 }
 
+static void
+fill_session(VirtViewerFile *file, SpiceSession *session)
+{
+    g_return_if_fail(VIRT_VIEWER_IS_FILE(file));
+    g_return_if_fail(SPICE_IS_SESSION(session));
+
+    if (virt_viewer_file_is_set(file, "host")) {
+        gchar *val = virt_viewer_file_get_host(file);
+        g_object_set(G_OBJECT(session), "host", val, NULL);
+        g_free(val);
+    }
+
+    if (virt_viewer_file_is_set(file, "port")) {
+        gchar *port = g_strdup_printf("%d", virt_viewer_file_get_port(file));
+        g_object_set(G_OBJECT(session), "port", port, NULL);
+        g_free(port);
+    }
+    if (virt_viewer_file_is_set(file, "tls-port")) {
+        gchar *tls_port = g_strdup_printf("%d", virt_viewer_file_get_tls_port(file));
+        g_object_set(G_OBJECT(session), "tls-port", tls_port, NULL);
+        g_free(tls_port);
+    }
+    if (virt_viewer_file_is_set(file, "password")) {
+        gchar *val = virt_viewer_file_get_password(file);
+        g_object_set(G_OBJECT(session), "password", val, NULL);
+        g_free(val);
+    }
+
+    if (virt_viewer_file_is_set(file, "tls-ciphers")) {
+        gchar *val = virt_viewer_file_get_tls_ciphers(file);
+        g_object_set(G_OBJECT(session), "ciphers", val, NULL);
+        g_free(val);
+    }
+
+    if (virt_viewer_file_is_set(file, "ca")) {
+        gchar *ca = virt_viewer_file_get_ca(file);
+        g_return_if_fail(ca != NULL);
+
+        GByteArray *ba = g_byte_array_new_take((guint8 *)ca, strlen(ca) + 1);
+        g_object_set(G_OBJECT(session), "ca", ba, NULL);
+        g_byte_array_unref(ba);
+    }
+
+    if (virt_viewer_file_is_set(file, "host-subject")) {
+        gchar *val = virt_viewer_file_get_host_subject(file);
+        g_object_set(G_OBJECT(session), "cert-subject", val, NULL);
+        g_free(val);
+    }
+
+    if (virt_viewer_file_is_set(file, "proxy")) {
+        gchar *val = virt_viewer_file_get_proxy(file);
+        g_object_set(G_OBJECT(session), "proxy", val, NULL);
+        g_free(val);
+    }
+
+    if (virt_viewer_file_is_set(file, "enable-smartcard")) {
+        g_object_set(G_OBJECT(session),
+                     "enable-smartcard", virt_viewer_file_get_enable_smartcard(file), NULL);
+    }
+
+    if (virt_viewer_file_is_set(file, "enable-usbredir")) {
+        g_object_set(G_OBJECT(session),
+                     "enable-usbredir", virt_viewer_file_get_enable_usbredir(file), NULL);
+    }
+
+    if (virt_viewer_file_is_set(file, "color-depth")) {
+        g_object_set(G_OBJECT(session),
+                     "color-depth", virt_viewer_file_get_color_depth(file), NULL);
+    }
+
+    if (virt_viewer_file_is_set(file, "disable-effects")) {
+        gchar **disabled = virt_viewer_file_get_disable_effects(file, NULL);
+        g_object_set(G_OBJECT(session), "disable-effects", disabled, NULL);
+        g_strfreev(disabled);
+    }
+
+    if (virt_viewer_file_is_set(file, "enable-usb-autoshare")) {
+        gboolean enabled = virt_viewer_file_get_enable_usb_autoshare(file);
+        SpiceGtkSession *gtk = spice_gtk_session_get(session);
+        g_object_set(G_OBJECT(gtk), "auto-usbredir", enabled, NULL);
+    }
+
+    if (virt_viewer_file_is_set(file, "secure-channels")) {
+        gchar **channels = virt_viewer_file_get_secure_channels(file, NULL);
+        g_object_set(G_OBJECT(session), "secure-channels", channels, NULL);
+        g_strfreev(channels);
+    }
+
+    if (virt_viewer_file_is_set(file, "disable-channels")) {
+        DEBUG_LOG("FIXME: disable-channels is not supported atm");
+    }
+}
+
 static gboolean
 virt_viewer_session_spice_open_uri(VirtViewerSession *session,
-                                   const gchar *uri)
+                                   const gchar *uri, GError **error)
 {
     VirtViewerSessionSpice *self = VIRT_VIEWER_SESSION_SPICE(session);
+    VirtViewerFile *file = virt_viewer_session_get_file(session);
+    VirtViewerApp *app = virt_viewer_session_get_app(session);
 
     g_return_val_if_fail(self != NULL, FALSE);
     g_return_val_if_fail(self->priv->session != NULL, FALSE);
 
-    g_object_set(self->priv->session, "uri", uri, NULL);
+    if (file) {
+        fill_session(file, self->priv->session);
+        if (!virt_viewer_file_fill_app(file, app, error))
+            return FALSE;
+    } else {
+        g_object_set(self->priv->session, "uri", uri, NULL);
+    }
 
     return spice_session_connect(self->priv->session);
 }
@@ -303,6 +412,7 @@ virt_viewer_session_spice_main_channel_event(SpiceChannel *channel G_GNUC_UNUSED
     switch (event) {
     case SPICE_CHANNEL_OPENED:
         DEBUG_LOG("main channel: opened");
+        g_signal_emit_by_name(session, "session-connected");
         break;
     case SPICE_CHANNEL_CLOSED:
         DEBUG_LOG("main channel: closed");
@@ -326,7 +436,6 @@ virt_viewer_session_spice_main_channel_event(SpiceChannel *channel G_GNUC_UNUSED
             gboolean openfd;
 
             g_object_set(self->priv->session, "password", password, NULL);
-            g_free(password);
             g_object_get(self->priv->session, "client-sockets", &openfd, NULL);
 
             if (openfd)
@@ -412,15 +521,8 @@ agent_connected_changed(SpiceChannel *cmain G_GNUC_UNUSED,
 {
     // this will force refresh of application menu
     g_signal_emit_by_name(self, "session-display-updated");
-}
 
-static void
-agent_connected_fullscreen_auto_conf(SpiceChannel *cmain,
-                                     GParamSpec *pspec G_GNUC_UNUSED,
-                                     VirtViewerSessionSpice *self)
-{
-    if (virt_viewer_session_spice_fullscreen_auto_conf(self))
-        g_signal_handlers_disconnect_by_func(cmain, agent_connected_fullscreen_auto_conf, self);
+    virt_viewer_session_spice_fullscreen_auto_conf(self);
 }
 
 static void
@@ -470,7 +572,6 @@ virt_viewer_session_spice_display_monitors(SpiceChannel *channel,
         }
 
         g_object_freeze_notify(G_OBJECT(display));
-        virt_viewer_display_set_enabled(VIRT_VIEWER_DISPLAY(display), FALSE);
         virt_viewer_session_add_display(VIRT_VIEWER_SESSION(self),
                                         VIRT_VIEWER_DISPLAY(display));
     }
@@ -522,10 +623,7 @@ virt_viewer_session_spice_channel_new(SpiceSession *s,
         self->priv->main_channel = SPICE_MAIN_CHANNEL(channel);
 
         g_signal_connect(channel, "notify::agent-connected", G_CALLBACK(agent_connected_changed), self);
-        g_signal_connect(channel, "notify::agent-connected", G_CALLBACK(agent_connected_fullscreen_auto_conf), self);
-        agent_connected_fullscreen_auto_conf(channel, NULL, self);
-
-        g_signal_emit_by_name(session, "session-connected");
+        virt_viewer_session_spice_fullscreen_auto_conf(self);
     }
 
     if (SPICE_IS_DISPLAY_CHANNEL(channel)) {
@@ -565,12 +663,18 @@ virt_viewer_session_spice_fullscreen_auto_conf(VirtViewerSessionSpice *self)
 
     DEBUG_LOG("Checking full screen auto-conf");
     g_object_get(app, "fullscreen-auto-conf", &auto_conf, NULL);
-    if (!auto_conf)
-        return TRUE;
-
-    if (cmain == NULL)
+    if (!auto_conf) {
+        DEBUG_LOG("auto-conf disabled");
         return FALSE;
-
+    }
+    if (!virt_viewer_app_get_fullscreen(app)) {
+        DEBUG_LOG("app is not in full screen");
+        return FALSE;
+    }
+    if (cmain == NULL) {
+        DEBUG_LOG("no main channel yet");
+        return FALSE;
+    }
     g_object_get(cmain, "agent-connected", &agent_connected, NULL);
     if (!agent_connected) {
         DEBUG_LOG("Agent not connected, skipping autoconf");
