@@ -28,9 +28,18 @@
 
 extern gboolean doDebug;
 
+enum {
+    VIRT_VIEWER_ERROR_FAILED,
+};
+
 #define DEBUG_LOG(s, ...) do { if (doDebug) g_debug(s, ## __VA_ARGS__); } while (0)
 #define ARRAY_CARDINALITY(Array) (sizeof (Array) / sizeof *(Array))
 
+#define VIRT_VIEWER_ERROR virt_viewer_error_quark ()
+
+GQuark virt_viewer_error_quark(void);
+
+void virt_viewer_util_init(const char *appname);
 
 GtkBuilder *virt_viewer_util_load_ui(const char *name);
 int virt_viewer_util_extract_host(const char *uristr,
@@ -46,6 +55,9 @@ gulong virt_viewer_signal_connect_object(gpointer instance,
                                          gpointer gobject,
                                          GConnectFlags connect_flags);
 
+gchar* spice_hotkey_to_gtk_accelerator(const gchar *key);
+gint virt_viewer_compare_version(const gchar *s1, const gchar *s2);
+
 #endif
 
 /*
