@@ -61,7 +61,7 @@ void virt_viewer_window_menu_view_release_cursor(GtkWidget *menu, VirtViewerWind
 /* Internal methods */
 static void virt_viewer_window_enable_modifiers(VirtViewerWindow *self);
 static void virt_viewer_window_disable_modifiers(VirtViewerWindow *self);
-static void virt_viewer_window_resize(VirtViewerWindow *self);
+static void virt_viewer_window_resize(VirtViewerWindow *self, gboolean keep_win_size);
 static void virt_viewer_window_toolbar_setup(VirtViewerWindow *self);
 static GtkMenu* virt_viewer_window_get_keycombo_menu(VirtViewerWindow *self);
 
@@ -100,6 +100,7 @@ struct _VirtViewerWindowPrivate {
     gboolean grabbed;
     gboolean before_saved;
     GdkRectangle before_fullscreen;
+    gint fullscreen_monitor;
     gboolean desktop_resize_pending;
 
     gint zoomlevel;
@@ -346,7 +347,7 @@ virt_viewer_window_desktop_resize(VirtViewerDisplay *display G_GNUC_UNUSED,
         self->priv->desktop_resize_pending = TRUE;
         return;
     }
-    virt_viewer_window_resize(self);
+    virt_viewer_window_resize(self, FALSE);
 }
 
 
@@ -354,46 +355,37 @@ G_MODULE_EXPORT void
 virt_viewer_window_menu_view_zoom_out(GtkWidget *menu G_GNUC_UNUSED,
                                       VirtViewerWindow *self)
 {
-    VirtViewerWindowPrivate *priv = self->priv;
-
-    if (priv->zoomlevel > 10)
-        priv->zoomlevel -= 10;
-
-    if (!priv->display)
-        return;
-
-    gtk_window_resize(GTK_WINDOW(priv->window), 1, 1);
-    if (priv->display)
-        virt_viewer_display_set_zoom_level(VIRT_VIEWER_DISPLAY(priv->display), priv->zoomlevel);
+    virt_viewer_window_set_zoom_level(self, self->priv->zoomlevel - 10);
 }
 
 G_MODULE_EXPORT void
 virt_viewer_window_menu_view_zoom_in(GtkWidget *menu G_GNUC_UNUSED,
                                      VirtViewerWindow *self)
 {
-    VirtViewerWindowPrivate *priv = self->priv;
-
-    if (priv->zoomlevel < 400)
-        priv->zoomlevel += 10;
-
-    if (!priv->display)
-        return;
-
-    gtk_window_resize(GTK_WINDOW(priv->window), 1, 1);
-    if (priv->display)
-        virt_viewer_display_set_zoom_level(VIRT_VIEWER_DISPLAY(priv->display), priv->zoomlevel);
+    virt_viewer_window_set_zoom_level(self, self->priv->zoomlevel + 10);
 }
 
 G_MODULE_EXPORT void
 virt_viewer_window_menu_view_zoom_reset(GtkWidget *menu G_GNUC_UNUSED,
                                         VirtViewerWindow *self)
 {
-    VirtViewerWindowPrivate *priv = self->priv;
-    gtk_window_resize(GTK_WINDOW(priv->window), 1, 1);
-    priv->zoomlevel = 100;
+    virt_viewer_window_set_zoom_level(self, 100);
+}
 
-    if (priv->display)
-        virt_viewer_display_set_zoom_level(VIRT_VIEWER_DISPLAY(priv->display), priv->zoomlevel);
+/* Kick GtkWindow to tell it to adjust to our new widget sizes */
+static void
+virt_viewer_window_queue_resize(VirtViewerWindow *self)
+{
+    VirtViewerWindowPrivate *priv = self->priv;
+#if GTK_CHECK_VERSION(3, 0, 0)
+    GtkRequisition nat;
+
+    gtk_window_set_default_size(GTK_WINDOW(priv->window), -1, -1);
+    gtk_widget_get_preferred_size(GTK_WIDGET(priv->window), NULL, &nat);
+    gtk_window_resize(GTK_WINDOW(priv->window), nat.width, nat.height);
+#else
+    gtk_window_resize(GTK_WINDOW(priv->window), 1, 1);
+#endif
 }
 
 /*
@@ -403,7 +395,7 @@ virt_viewer_window_menu_view_zoom_reset(GtkWidget *menu G_GNUC_UNUSED,
  * scale down to fit, maintaining aspect ratio
  */
 static void
-virt_viewer_window_resize(VirtViewerWindow *self)
+virt_viewer_window_resize(VirtViewerWindow *self, gboolean keep_win_size)
 {
     GdkRectangle fullscreen;
     GdkScreen *screen;
@@ -423,8 +415,6 @@ virt_viewer_window_resize(VirtViewerWindow *self)
         return;
     }
 
-    gtk_window_resize(GTK_WINDOW(priv->window), 1, 1);
-
     virt_viewer_display_get_desktop_size(VIRT_VIEWER_DISPLAY(priv->display),
                                          &desktopWidth, &desktopHeight);
 
@@ -464,6 +454,30 @@ virt_viewer_window_resize(VirtViewerWindow *self)
 
     virt_viewer_display_set_desktop_size(VIRT_VIEWER_DISPLAY(priv->display),
                                          width, height);
+
+    if (!keep_win_size)
+        virt_viewer_window_queue_resize(self);
+}
+
+static void
+virt_viewer_window_move_to_monitor(VirtViewerWindow *self)
+{
+    VirtViewerWindowPrivate *priv = self->priv;
+    GdkRectangle mon;
+    gint n = priv->fullscreen_monitor;
+
+    if (n == -1 || !priv->fullscreen)
+        return;
+
+    gdk_screen_get_monitor_geometry(gdk_screen_get_default(), n, &mon);
+    gtk_window_move(GTK_WINDOW(priv->window), mon.x, mon.y);
+#ifdef G_OS_WIN32
+    /* FIXME: on windows, fullscreen doesn't always hide the taskbar
+       See https://bugzilla.gnome.org/show_bug.cgi?id=652049 */
+    gtk_widget_set_size_request(GTK_WIDGET(priv->window),
+                                mon.width,
+                                mon.height);
+#endif
 }
 
 void
@@ -478,6 +492,9 @@ virt_viewer_window_leave_fullscreen(VirtViewerWindow *self)
 
     gtk_check_menu_item_set_active(check, FALSE);
     priv->fullscreen = FALSE;
+    priv->fullscreen_monitor = -1;
+    if (priv->display)
+        virt_viewer_display_set_monitor(priv->display, -1);
     ViewAutoDrawer_SetActive(VIEW_AUTODRAWER(priv->layout), FALSE);
     gtk_widget_show(menu);
     gtk_widget_hide(priv->toolbar);
@@ -493,18 +510,29 @@ virt_viewer_window_leave_fullscreen(VirtViewerWindow *self)
         gtk_window_resize(GTK_WINDOW(priv->window),
                           priv->before_fullscreen.width,
                           priv->before_fullscreen.height);
-        priv->before_saved = FALSE;
+    } else {
+#ifdef G_OS_WIN32
+        /* win32 window manager isn't smart enough to place
+         * the window so that titlebar would be visible */
+        gtk_window_maximize(GTK_WINDOW(priv->window));
+#else
+        virt_viewer_display_queue_resize(priv->display);
+#endif
     }
 }
 
 void
-virt_viewer_window_enter_fullscreen(VirtViewerWindow *self, gboolean move, gint x, gint y)
+virt_viewer_window_enter_fullscreen(VirtViewerWindow *self, gint monitor)
 {
     VirtViewerWindowPrivate *priv = self->priv;
     GtkWidget *menu = GTK_WIDGET(gtk_builder_get_object(priv->builder, "top-menu"));
     GtkCheckMenuItem *check = GTK_CHECK_MENU_ITEM(gtk_builder_get_object(priv->builder, "menu-view-fullscreen"));
 
-    if (!priv->before_saved) {
+    if (priv->fullscreen)
+        return;
+    priv->fullscreen = TRUE;
+
+    if (gtk_widget_get_realized(priv->window)) {
         gtk_window_get_position(GTK_WINDOW(priv->window),
                                 &priv->before_fullscreen.x,
                                 &priv->before_fullscreen.y);
@@ -514,28 +542,18 @@ virt_viewer_window_enter_fullscreen(VirtViewerWindow *self, gboolean move, gint
         priv->before_saved = TRUE;
     }
 
-    if (priv->fullscreen)
-        return;
-    priv->fullscreen = TRUE;
-
     gtk_check_menu_item_set_active(check, TRUE);
     gtk_widget_hide(menu);
     gtk_widget_show(priv->toolbar);
     ViewAutoDrawer_SetActive(VIEW_AUTODRAWER(priv->layout), TRUE);
     ViewAutoDrawer_Close(VIEW_AUTODRAWER(priv->layout));
 
-    /* g_debug("enter fullscreen move:%d %d+%d", move, x, y); */
-    if (move)
-        gtk_window_move(GTK_WINDOW(priv->window), x, y);
+    priv->fullscreen_monitor = monitor;
+    if (priv->display)
+        virt_viewer_display_set_monitor(priv->display, monitor);
+    virt_viewer_window_move_to_monitor(self);
 
     gtk_window_fullscreen(GTK_WINDOW(priv->window));
-#ifdef G_OS_WIN32
-    /* on windows, fullscreen doesn't always hide the taskbar
-       See https://bugzilla.gnome.org/show_bug.cgi?id=652049 */
-    gtk_widget_set_size_request(GTK_WIDGET(priv->window),
-                                gdk_screen_width(),
-                                gdk_screen_height());
-#endif
 }
 
 #define MAX_KEY_COMBO 3
@@ -546,21 +564,21 @@ struct        keyComboDef {
 };
 
 static const struct keyComboDef keyCombos[] = {
-    { { GDK_Control_L, GDK_Alt_L, GDK_Delete }, 3, "Ctrl+Alt+_Del"},
-    { { GDK_Control_L, GDK_Alt_L, GDK_BackSpace }, 3, "Ctrl+Alt+_Backspace"},
+    { { GDK_Control_L, GDK_Alt_L, GDK_Delete }, 3, N_("Ctrl+Alt+_Del")},
+    { { GDK_Control_L, GDK_Alt_L, GDK_BackSpace }, 3, N_("Ctrl+Alt+_Backspace")},
     { {}, 0, "" },
-    { { GDK_Control_L, GDK_Alt_L, GDK_F1 }, 3, "Ctrl+Alt+F_1"},
-    { { GDK_Control_L, GDK_Alt_L, GDK_F2 }, 3, "Ctrl+Alt+F_2"},
-    { { GDK_Control_L, GDK_Alt_L, GDK_F3 }, 3, "Ctrl+Alt+F_3"},
-    { { GDK_Control_L, GDK_Alt_L, GDK_F4 }, 3, "Ctrl+Alt+F_4"},
-    { { GDK_Control_L, GDK_Alt_L, GDK_F5 }, 3, "Ctrl+Alt+F_5"},
-    { { GDK_Control_L, GDK_Alt_L, GDK_F6 }, 3, "Ctrl+Alt+F_6"},
-    { { GDK_Control_L, GDK_Alt_L, GDK_F7 }, 3, "Ctrl+Alt+F_7"},
-    { { GDK_Control_L, GDK_Alt_L, GDK_F8 }, 3, "Ctrl+Alt+F_8"},
-    { { GDK_Control_L, GDK_Alt_L, GDK_F9 }, 3, "Ctrl+Alt+F_9"},
-    { { GDK_Control_L, GDK_Alt_L, GDK_F10 }, 3, "Ctrl+Alt+F1_0"},
-    { { GDK_Control_L, GDK_Alt_L, GDK_F11 }, 3, "Ctrl+Alt+F11"},
-    { { GDK_Control_L, GDK_Alt_L, GDK_F12 }, 3, "Ctrl+Alt+F12"},
+    { { GDK_Control_L, GDK_Alt_L, GDK_F1 }, 3, N_("Ctrl+Alt+F_1")},
+    { { GDK_Control_L, GDK_Alt_L, GDK_F2 }, 3, N_("Ctrl+Alt+F_2")},
+    { { GDK_Control_L, GDK_Alt_L, GDK_F3 }, 3, N_("Ctrl+Alt+F_3")},
+    { { GDK_Control_L, GDK_Alt_L, GDK_F4 }, 3, N_("Ctrl+Alt+F_4")},
+    { { GDK_Control_L, GDK_Alt_L, GDK_F5 }, 3, N_("Ctrl+Alt+F_5")},
+    { { GDK_Control_L, GDK_Alt_L, GDK_F6 }, 3, N_("Ctrl+Alt+F_6")},
+    { { GDK_Control_L, GDK_Alt_L, GDK_F7 }, 3, N_("Ctrl+Alt+F_7")},
+    { { GDK_Control_L, GDK_Alt_L, GDK_F8 }, 3, N_("Ctrl+Alt+F_8")},
+    { { GDK_Control_L, GDK_Alt_L, GDK_F9 }, 3, N_("Ctrl+Alt+F_9")},
+    { { GDK_Control_L, GDK_Alt_L, GDK_F10 }, 3, N_("Ctrl+Alt+F1_0")},
+    { { GDK_Control_L, GDK_Alt_L, GDK_F11 }, 3, N_("Ctrl+Alt+F11")},
+    { { GDK_Control_L, GDK_Alt_L, GDK_F12 }, 3, N_("Ctrl+Alt+F12")},
     { {}, 0, "" },
     { { GDK_Print }, 1, "_PrintScreen"},
 };
@@ -574,8 +592,10 @@ virt_viewer_window_menu_send(GtkWidget *menu,
     const char *text = gtk_label_get_label(GTK_LABEL(label));
     VirtViewerWindowPrivate *priv = self->priv;
 
+    g_return_if_fail(priv->display != NULL);
+
     for (i = 0 ; i < G_N_ELEMENTS(keyCombos) ; i++) {
-        if (!strcmp(text, keyCombos[i].label)) {
+        if (!strcmp(text, _(keyCombos[i].label))) {
             DEBUG_LOG("Sending key combo %s", gtk_label_get_text(GTK_LABEL(label)));
             virt_viewer_display_send_keys(VIRT_VIEWER_DISPLAY(priv->display),
                                           keyCombos[i].keys,
@@ -590,6 +610,7 @@ static GtkMenu*
 virt_viewer_window_get_keycombo_menu(VirtViewerWindow *self)
 {
     gint i;
+    VirtViewerWindowPrivate *priv = self->priv;
     GtkMenu *menu = GTK_MENU(gtk_menu_new());
 
     for (i = 0 ; i < G_N_ELEMENTS(keyCombos) ; i++) {
@@ -603,6 +624,7 @@ virt_viewer_window_get_keycombo_menu(VirtViewerWindow *self)
         gtk_container_add(GTK_CONTAINER(menu), item);
     }
 
+    gtk_menu_attach_to_widget(menu, GTK_WIDGET(priv->window), NULL);
     gtk_widget_show_all(GTK_WIDGET(menu));
     return g_object_ref_sink(menu);
 }
@@ -679,7 +701,7 @@ virt_viewer_window_delete(GtkWidget *src G_GNUC_UNUSED,
                           VirtViewerWindow *self)
 {
     DEBUG_LOG("Window closed");
-    virt_viewer_app_window_set_visible(self->priv->app, self, FALSE);
+    virt_viewer_app_maybe_quit(self->priv->app, self);
     return TRUE;
 }
 
@@ -741,12 +763,54 @@ virt_viewer_window_menu_view_resize(GtkWidget *menu,
 
     if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menu))) {
         priv->auto_resize = TRUE;
-        virt_viewer_window_resize(self);
+        virt_viewer_window_resize(self, TRUE);
     } else {
         priv->auto_resize = FALSE;
     }
 
-    virt_viewer_display_set_auto_resize(priv->display, priv->auto_resize);
+    if (priv->display)
+        virt_viewer_display_set_auto_resize(priv->display, priv->auto_resize);
+}
+
+static void add_if_writable (GdkPixbufFormat *data, GHashTable *formats)
+{
+    if (gdk_pixbuf_format_is_writable(data)) {
+        gchar **extensions;
+        gchar **it;
+        extensions = gdk_pixbuf_format_get_extensions(data);
+        for (it = extensions; *it != NULL; it++) {
+            g_hash_table_insert(formats, g_strdup(*it), data);
+        }
+        g_strfreev(extensions);
+    }
+}
+
+static GHashTable *init_image_formats(void)
+{
+    GHashTable *format_map;
+    GSList *formats = gdk_pixbuf_get_formats();
+
+    format_map = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
+    g_slist_foreach(formats, (GFunc)add_if_writable, format_map);
+    g_slist_free (formats);
+
+    return format_map;
+}
+
+static GdkPixbufFormat *get_image_format(const char *filename)
+{
+    static GOnce image_formats_once = G_ONCE_INIT;
+    const char *ext;
+
+    g_once(&image_formats_once, (GThreadFunc)init_image_formats, NULL);
+
+    ext = strrchr(filename, '.');
+    if (ext == NULL)
+        return NULL;
+
+    ext++; /* skip '.' */
+
+    return g_hash_table_lookup(image_formats_once.retval, ext);
 }
 
 static void
@@ -755,9 +819,27 @@ virt_viewer_window_save_screenshot(VirtViewerWindow *self,
 {
     VirtViewerWindowPrivate *priv = self->priv;
     GdkPixbuf *pix = virt_viewer_display_get_pixbuf(VIRT_VIEWER_DISPLAY(priv->display));
+    GdkPixbufFormat *format = get_image_format(file);
+
+    if (format == NULL) {
+        g_debug("unknown file extension, falling back to png");
+        if (!g_str_has_suffix(file, ".png")) {
+            char *png_filename;
+            png_filename = g_strconcat(file, ".png", NULL);
+            gdk_pixbuf_save(pix, png_filename, "png", NULL,
+                            "tEXt::Generator App", PACKAGE, NULL);
+            g_free(png_filename);
+        } else {
+            gdk_pixbuf_save(pix, file, "png", NULL,
+                            "tEXt::Generator App", PACKAGE, NULL);
+        }
+    } else {
+        char *type = gdk_pixbuf_format_get_name(format);
+        g_debug("saving to %s", type);
+        gdk_pixbuf_save(pix, file, type, NULL, NULL);
+        g_free(type);
+    }
 
-    gdk_pixbuf_save(pix, file, "png", NULL,
-                    "tEXt::Generator App", PACKAGE, NULL);
     g_object_unref(pix);
 }
 
@@ -767,6 +849,7 @@ virt_viewer_window_menu_file_screenshot(GtkWidget *menu G_GNUC_UNUSED,
 {
     GtkWidget *dialog;
     VirtViewerWindowPrivate *priv = self->priv;
+    const char *image_dir;
 
     g_return_if_fail(priv->display != NULL);
 
@@ -779,9 +862,10 @@ virt_viewer_window_menu_file_screenshot(GtkWidget *menu G_GNUC_UNUSED,
     gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER (dialog), TRUE);
     gtk_window_set_transient_for(GTK_WINDOW(dialog),
                                  GTK_WINDOW(self->priv->window));
-
-    //gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), default_folder_for_saving);
-    //gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), "Screenshot");
+    image_dir = g_get_user_special_dir(G_USER_DIRECTORY_PICTURES);
+    if (image_dir != NULL)
+        gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER (dialog), image_dir);
+    gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER (dialog), _("Screenshot"));
 
     if (gtk_dialog_run(GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
         char *filename;
@@ -820,6 +904,7 @@ G_MODULE_EXPORT void
 virt_viewer_window_menu_view_release_cursor(GtkWidget *menu G_GNUC_UNUSED,
                                             VirtViewerWindow *self)
 {
+    g_return_if_fail(self->priv->display != NULL);
     virt_viewer_display_release_cursor(VIRT_VIEWER_DISPLAY(self->priv->display));
 }
 
@@ -830,7 +915,7 @@ virt_viewer_window_menu_help_about(GtkWidget *menu G_GNUC_UNUSED,
     GtkBuilder *about = virt_viewer_util_load_ui("virt-viewer-about.xml");
 
     GtkWidget *dialog = GTK_WIDGET(gtk_builder_get_object(about, "about"));
-    gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(dialog), VERSION);
+    gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(dialog), VERSION BUILDID);
 
     gtk_window_set_transient_for(GTK_WINDOW(dialog),
                                  GTK_WINDOW(self->priv->window));
@@ -955,10 +1040,10 @@ virt_viewer_window_update_title(VirtViewerWindow *self)
 
     if (priv->grabbed) {
         gchar *label;
+        GtkAccelKey key;
 
-        if (virt_viewer_app_get_enable_accel(priv->app)) {
-            GtkAccelKey key;
-            gtk_accel_map_lookup_entry("<virt-viewer>/view/release-cursor", &key);
+        if (virt_viewer_app_get_enable_accel(priv->app)
+                && gtk_accel_map_lookup_entry("<virt-viewer>/view/release-cursor", &key)) {
             label = gtk_accelerator_get_label(key.accel_key, key.accel_mods);
         } else {
             label = g_strdup(_("Ctrl+Alt"));
@@ -1018,6 +1103,14 @@ display_show_hint(VirtViewerDisplay *display,
     gtk_widget_set_sensitive(GTK_WIDGET(gtk_builder_get_object(self->priv->builder, "menu-file-screenshot")), hint);
     gtk_widget_set_sensitive(self->priv->toolbar_send_key, hint);
 }
+static gboolean
+window_key_pressed (GtkWidget *widget G_GNUC_UNUSED,
+                    GdkEvent  *event,
+                    GtkWidget *display)
+{
+    gtk_widget_grab_focus(display);
+    return gtk_widget_event(display, event);
+}
 
 void
 virt_viewer_window_set_display(VirtViewerWindow *self, VirtViewerDisplay *display)
@@ -1039,9 +1132,14 @@ virt_viewer_window_set_display(VirtViewerWindow *self, VirtViewerDisplay *displa
 
         virt_viewer_display_set_zoom_level(VIRT_VIEWER_DISPLAY(priv->display), priv->zoomlevel);
         virt_viewer_display_set_auto_resize(VIRT_VIEWER_DISPLAY(priv->display), priv->auto_resize);
+        virt_viewer_display_set_monitor(VIRT_VIEWER_DISPLAY(priv->display), priv->fullscreen_monitor);
 
         gtk_widget_show_all(GTK_WIDGET(display));
         gtk_notebook_append_page(GTK_NOTEBOOK(priv->notebook), GTK_WIDGET(display), NULL);
+
+        virt_viewer_signal_connect_object(priv->window, "key-press-event",
+                                          G_CALLBACK(window_key_pressed), display, 0);
+
         /* switch back to non-display if not ready */
         if (!(virt_viewer_display_get_show_hint(display) &
               VIRT_VIEWER_DISPLAY_SHOW_HINT_READY))
@@ -1071,9 +1169,11 @@ virt_viewer_window_show(VirtViewerWindow *self)
         virt_viewer_display_set_enabled(self->priv->display, TRUE);
 
     if (self->priv->desktop_resize_pending) {
-        virt_viewer_window_resize(self);
+        virt_viewer_window_resize(self, FALSE);
         self->priv->desktop_resize_pending = FALSE;
     }
+
+    virt_viewer_window_move_to_monitor(self);
 }
 
 void
@@ -1094,10 +1194,23 @@ virt_viewer_window_hide(VirtViewerWindow *self)
 void
 virt_viewer_window_set_zoom_level(VirtViewerWindow *self, gint zoom_level)
 {
-    g_return_if_fail(VIRT_VIEWER_IS_WINDOW(self));
+    VirtViewerWindowPrivate *priv;
 
-    /* FIXME: turn into a dynamic property */
-    self->priv->zoomlevel = zoom_level;
+    g_return_if_fail(VIRT_VIEWER_IS_WINDOW(self));
+    priv = self->priv;
+
+    if (zoom_level < 10)
+        zoom_level = 10;
+    if (zoom_level > 400)
+        zoom_level = 400;
+    priv->zoomlevel = zoom_level;
+
+    if (!priv->display)
+        return;
+
+    virt_viewer_display_set_zoom_level(VIRT_VIEWER_DISPLAY(priv->display), priv->zoomlevel);
+
+    virt_viewer_window_queue_resize(self);
 }
 
 gint virt_viewer_window_get_zoom_level(VirtViewerWindow *self)
