@@ -63,7 +63,7 @@ G_DEFINE_TYPE (VirtViewer, virt_viewer, VIRT_VIEWER_TYPE_APP)
 #define GET_PRIVATE(o)                                                        \
     (G_TYPE_INSTANCE_GET_PRIVATE ((o), VIRT_VIEWER_TYPE, VirtViewerPrivate))
 
-static int virt_viewer_initial_connect(VirtViewerApp *self);
+static gboolean virt_viewer_initial_connect(VirtViewerApp *self, GError **error);
 static gboolean virt_viewer_open_connection(VirtViewerApp *self, int *fd);
 static void virt_viewer_deactivated(VirtViewerApp *self);
 static gboolean virt_viewer_start(VirtViewerApp *self);
@@ -282,6 +282,27 @@ virt_viewer_extract_xpath_string(const gchar *xmldesc,
     return port;
 }
 
+
+static gboolean
+virt_viewer_replace_host(const gchar *host)
+{
+    GInetAddress *addr;
+    gboolean ret;
+
+    if (!host)
+        return TRUE;
+
+    addr = g_inet_address_new_from_string(host);
+
+    if (!addr) /* Parsing error means it was probably a hostname */
+        return FALSE;
+
+    ret = g_inet_address_get_is_any(addr);
+    g_object_unref(addr);
+
+    return ret;
+}
+
 static gboolean
 virt_viewer_extract_connect_info(VirtViewer *self,
                                  virDomainPtr dom)
@@ -349,15 +370,21 @@ virt_viewer_extract_connect_info(VirtViewer *self,
     /* If the XML listen attribute shows a wildcard address, we need to
      * throw that away since you obviously can't 'connect(2)' to that
      * from a remote host. Instead we fallback to the hostname used in
-     * the libvirt URI. This isn't perfect but it is better than nothing
+     * the libvirt URI. This isn't perfect but it is better than nothing.
+     * If the transport is SSH, fallback to localhost as the connection
+     * will be made from the remote end of the ssh connection.
      */
-    if (!ghost ||
-        (strcmp(ghost, "0.0.0.0") == 0 ||
-         strcmp(ghost, "::") == 0)) {
+    if (virt_viewer_replace_host(ghost)) {
+        gchar *replacement_host = NULL;
+        if (g_strcmp0(transport, "ssh") == 0) {
+            replacement_host = g_strdup("localhost");
+        } else {
+            replacement_host = g_strdup(host);
+        }
         DEBUG_LOG("Guest graphics listen '%s' is NULL or a wildcard, replacing with '%s'",
-                  ghost ? ghost : "", host);
+                  ghost ? ghost : "", replacement_host);
         g_free(ghost);
-        ghost = g_strdup(host);
+        ghost = replacement_host;
     }
 
     virt_viewer_app_set_connect_info(app, host, ghost, gport, gtlsport,transport, unixsock, user, port, NULL);
@@ -379,7 +406,7 @@ virt_viewer_extract_connect_info(VirtViewer *self,
     return retval;
 }
 
-static int
+static gboolean
 virt_viewer_update_display(VirtViewer *self, virDomainPtr dom)
 {
     VirtViewerPrivate *priv = self->priv;
@@ -390,17 +417,17 @@ virt_viewer_update_display(VirtViewer *self, virDomainPtr dom)
     priv->dom = dom;
     virDomainRef(priv->dom);
 
-    virt_viewer_app_trace(app, "Guest %s is running, determining display\n",
+    virt_viewer_app_trace(app, "Guest %s is running, determining display",
                           priv->domkey);
 
     g_object_set(app, "title", virDomainGetName(dom), NULL);
 
     if (!virt_viewer_app_has_session(app)) {
         if (!virt_viewer_extract_connect_info(self, dom))
-            return -1;
+            return FALSE;
     }
 
-    return 0;
+    return TRUE;
 }
 
 static gboolean
@@ -442,6 +469,7 @@ virt_viewer_domain_event(virConnectPtr conn G_GNUC_UNUSED,
 {
     VirtViewer *self = opaque;
     VirtViewerApp *app = VIRT_VIEWER_APP(self);
+    GError *error = NULL;
 
     DEBUG_LOG("Got domain event %d %d", event, detail);
 
@@ -455,28 +483,61 @@ virt_viewer_domain_event(virConnectPtr conn G_GNUC_UNUSED,
 
     case VIR_DOMAIN_EVENT_STARTED:
         virt_viewer_update_display(self, dom);
-        virt_viewer_app_activate(app);
+        virt_viewer_app_activate(app, &error);
+        if (error) {
+            /* we may want to consolidate error reporting in
+               app_activate() instead */
+            g_warning("%s", error->message);
+            g_clear_error(&error);
+        }
         break;
     }
 
     return 0;
 }
 
-static int
-virt_viewer_initial_connect(VirtViewerApp *app)
+static void
+virt_viewer_conn_event(virConnectPtr conn G_GNUC_UNUSED,
+                       int reason,
+                       void *opaque)
+{
+    VirtViewer *self = opaque;
+    VirtViewerApp *app = VIRT_VIEWER_APP(self);
+    VirtViewerPrivate *priv = self->priv;
+
+    DEBUG_LOG("Got connection event %d", reason);
+
+    virConnectClose(priv->conn);
+    priv->conn = NULL;
+
+    virt_viewer_app_start_reconnect_poll(app);
+}
+
+static int virt_viewer_connect(VirtViewerApp *app);
+
+static gboolean
+virt_viewer_initial_connect(VirtViewerApp *app, GError **error)
 {
     virDomainPtr dom = NULL;
     virDomainInfo info;
-    int ret = -1;
+    gboolean ret = FALSE;
     VirtViewer *self = VIRT_VIEWER(app);
     VirtViewerPrivate *priv = self->priv;
 
+    DEBUG_LOG("initial connect");
+
+    if (!priv->conn &&
+        virt_viewer_connect(app) < 0) {
+        virt_viewer_app_show_status(app, _("Waiting for libvirt to start"));
+        goto done;
+    }
+
     virt_viewer_app_show_status(app, _("Finding guest domain"));
     dom = virt_viewer_lookup_domain(self);
     if (!dom) {
         if (priv->waitvm) {
             virt_viewer_app_show_status(app, _("Waiting for guest domain to be created"));
-            virt_viewer_app_trace(app, "Guest %s does not yet exist, waiting for it to be created\n",
+            virt_viewer_app_trace(app, "Guest %s does not yet exist, waiting for it to be created",
                                   priv->domkey);
             goto done;
         } else {
@@ -497,26 +558,22 @@ virt_viewer_initial_connect(VirtViewerApp *app)
         virt_viewer_app_show_status(app, _("Waiting for guest domain to start"));
     } else {
         ret = virt_viewer_update_display(self, dom);
-        if (ret >= 0)
-            ret = VIRT_VIEWER_APP_CLASS(virt_viewer_parent_class)->initial_connect(app);
-        if (ret < 0) {
+        if (ret)
+            ret = VIRT_VIEWER_APP_CLASS(virt_viewer_parent_class)->initial_connect(app, error);
+        if (!ret) {
             if (priv->waitvm) {
                 virt_viewer_app_show_status(app, _("Waiting for guest domain to start server"));
-                virt_viewer_app_trace(app, "Guest %s has not activated its display yet, waiting for it to start\n",
+                virt_viewer_app_trace(app, "Guest %s has not activated its display yet, waiting for it to start",
                                       priv->domkey);
             } else {
                 DEBUG_LOG("Failed to activate viewer");
                 goto cleanup;
             }
-        } else if (ret == 0) {
-            DEBUG_LOG("Failed to activate viewer");
-            ret = -1;
-            goto cleanup;
         }
     }
 
  done:
-    ret = 0;
+    ret = TRUE;
  cleanup:
     if (dom)
         virDomainFree(dom);
@@ -591,9 +648,8 @@ virt_viewer_auth_libvirt_credentials(virConnectCredentialPtr cred,
     return ret;
 }
 
-
-static gboolean
-virt_viewer_start(VirtViewerApp *app)
+static int
+virt_viewer_connect(VirtViewerApp *app)
 {
     VirtViewer *self = VIRT_VIEWER(app);
     VirtViewerPrivate *priv = self->priv;
@@ -606,15 +662,14 @@ virt_viewer_start(VirtViewerApp *app)
         .cbdata = app,
     };
     int oflags = 0;
+    GError *error = NULL;
 
     if (!virt_viewer_app_get_attach(app))
         oflags |= VIR_CONNECT_RO;
 
-    virt_viewer_events_register();
+    DEBUG_LOG("connecting ...");
 
-    virSetErrorFunc(NULL, virt_viewer_error_func);
-
-    virt_viewer_app_trace(app, "Opening connection to libvirt with URI %s\n",
+    virt_viewer_app_trace(app, "Opening connection to libvirt with URI %s",
                           priv->uri ? priv->uri : "<null>");
     priv->conn = virConnectOpenAuth(priv->uri,
                                     //virConnectAuthPtrDefault,
@@ -623,11 +678,15 @@ virt_viewer_start(VirtViewerApp *app)
     if (!priv->conn) {
         virt_viewer_app_simple_message_dialog(app, _("Unable to connect to libvirt with URI %s"),
                                               priv->uri ? priv->uri : _("[none]"));
-        return FALSE;
+        return -1;
     }
 
-    if (virt_viewer_app_initial_connect(app) < 0)
-        return FALSE;
+    if (!virt_viewer_app_initial_connect(app, &error)) {
+        if (error)
+            g_warning(error->message);
+        g_clear_error(&error);
+        return -1;
+    }
 
     if (virConnectDomainEventRegister(priv->conn,
                                       virt_viewer_domain_event,
@@ -643,6 +702,26 @@ virt_viewer_start(VirtViewerApp *app)
         virt_viewer_app_start_reconnect_poll(app);
     }
 
+    if (virConnectRegisterCloseCallback(priv->conn,
+                                        virt_viewer_conn_event,
+                                        self,
+                                        NULL) < 0) {
+        DEBUG_LOG("Unable to register close callback on libvirt connection");
+    }
+
+    return 0;
+}
+
+static gboolean
+virt_viewer_start(VirtViewerApp *app)
+{
+    virt_viewer_events_register();
+
+    virSetErrorFunc(NULL, virt_viewer_error_func);
+
+    if (virt_viewer_connect(app) < 0)
+        return FALSE;
+
     return VIRT_VIEWER_APP_CLASS(virt_viewer_parent_class)->start(app);
 }
 
